import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  static targets = ['input', 'fileInfoContainer', 'dragAndDropArea'];

  connect() {
    this.hasSingleFile = this.inputTarget.getAttribute('multiple') === null
    this.files = [];
    this.inputTarget.addEventListener('change', this.handleInputChange.bind(this));
    this.fileInfoContainerTarget.addEventListener('click', this.handleFileContainerClick.bind(this));
  }

  disconnect() {
    this.inputTarget.removeEventListener('change', this.handleInputChange.bind(this));
    this.fileInfoContainerTarget.removeEventListener('click', this.handleFileContainerClick.bind(this));
  }

  handleInputChange(event) {
    event.preventDefault();

    if (this.hasSingleFile) { this.files = [] }
    const newFiles = Array.from(this.inputTarget.files);

    this.addUniqueFiles(newFiles);
    this.updateInputFiles();
    this.displayFileNames();
  }

  dropFile(event) {
    event.preventDefault();
    this.dragAndDropAreaTarget.classList.remove('bg-gray-200');
    const droppedFiles = Array.from(event.dataTransfer.files);

    this.addUniqueFiles(droppedFiles);
    this.updateInputFiles();
    this.displayFileNames();
  }

  updateInputFiles() {
    const dataTransfer = new DataTransfer();

    this.files.forEach(file => {
      dataTransfer.items.add(file);
    });

    this.inputTarget.files = dataTransfer.files;
    this.inputTarget.dispatchEvent(new Event('updatedFiles'));
  }

  addUniqueFiles(newFiles) {
    newFiles.forEach(file => {
      if (!this.files.some(existingFile => existingFile.name === file.name && existingFile.size === file.size)) {
        this.files.push(file);
      }
    });
  }

  displayFileNames() {
    this.fileInfoContainerTarget.innerHTML = '';

    this.files.forEach((file, index) => {
      const fileInfoDiv = document.createElement('div');
      fileInfoDiv.className = 'flex items-center justify-between px-4 py-2 first:pt-4 last:pb-4';

      const fileNameSpan = document.createElement('span');
      fileNameSpan.className = 'text-gray-700';
      fileNameSpan.textContent = file.name;
      fileInfoDiv.appendChild(fileNameSpan);

      const removeButton = document.createElement('button');
      removeButton.type = 'button';
      removeButton.className = 'text-spirit-red hover:text-red-700';
      removeButton.textContent = 'Supprimer';
      removeButton.dataset.action = 'remove';
      removeButton.dataset.index = index;
      fileInfoDiv.appendChild(removeButton);

      this.fileInfoContainerTarget.appendChild(fileInfoDiv);
    });
  }

  handleFileContainerClick(event) {
    const button = event.target.closest('button');
    if (button && button.dataset.action === 'remove') {
      this.removeFile(button.dataset.index);
    }
  }

  removeFile(index) {
    this.files.splice(index, 1);

    this.updateInputFiles();
    this.displayFileNames();
  }

  selectFile() {
    this.inputTarget.click();
  }

  highlight(event) {
    event.preventDefault();
    this.dragAndDropAreaTarget.classList.add('bg-gray-200');
  }
}
