import { Controller } from '@hotwired/stimulus';
import * as pdfjsLib from 'pdfjs-dist';
import { renderThumbnails } from 'helpers/pdf_thumbnails';
import { AnnotationService } from 'helpers/annotation_service';
import debounce from 'debounce';

const ZOOM = {
  MAX: 3,
  MIN: 0.5,
  STEP: 0.25,
};

const ANNOTATION_TYPES = {
  FREETEXT: 'freetext',
};

export default class extends Controller {
  static targets = [
    'content',
    'indexIndicator',
    'totalPagesIndicator',
    'zoomIndicator',
    'nextPageButton',
    'previousPageButton',
    'zoomInButton',
    'zoomOutButton',
    'thumbnailContainer',
    'annotationLayer',
    'editButton',
    'cancelButton',
    'loadingIndicator',
  ];

  static values = {
    src: String,
    index: { type: Number, default: 1 },
    zoom: { type: Number, default: 1 },
    totalPages: { type: Number, default: 1 },
    active: { type: String, default: 'border-spirit-red hover:border-spirit-red' },
    workerUrl: String,
    annotationMode: { type: String, default: 'none' },
    documentId: String,
    isRendering: { type: Boolean, default: false },
    pdfLoaded: { type: Boolean, default: false },
  };

  static THUMBNAIL_SIZE = 150;

  initialize() {
    this.annotationService = new AnnotationService(this.documentIdValue);
    this.annotations = [];

    this.debouncedSaveAnnotation = debounce(this.saveAnnotationToBackend.bind(this), 500);
  }

  connect() {
    this.loadPDF();
  }

  disconnect() {
    if (this.pdf) {
      this.pdf.destroy();
      this.pdf = null;
      this.annotations = [];
    }
  }

  async loadPDF() {
    this.showLoadingSpinner(true);

    pdfjsLib.GlobalWorkerOptions.workerSrc = this.workerUrlValue;

    try {
      this.pdf = await pdfjsLib.getDocument({
        enableAnnotations: true,
        url: this.srcValue,
        maxCanvasBytes: 10000000,
        disableFontFace: true,
      }).promise;

      this.totalPagesValue = this.pdf.numPages;
      this.totalPagesIndicatorTarget.textContent = this.totalPagesValue;
      await this.renderCurrentPage();
      this.loadAnnotations();
      await renderThumbnails.call(this, 'click->pdf-viewer#goToPage');
    } catch (error) {
      console.error('Error loading PDF:', error);
      this.showLoadingSpinner(false);
    }
  }

  async renderCurrentPage() {
    if (this.isRenderingValue) return;

    this.isRenderingValue = true;
    this.showLoadingSpinner(true);

    try {
      this.updateActiveThumbnail();

      const page = await this.pdf.getPage(this.indexValue);
      const viewport = page.getViewport({ scale: this.zoomValue });

      const canvas = this.contentTarget;
      const context = canvas.getContext('2d', { alpha: false });

      canvas.height = viewport.height;
      canvas.width = viewport.width;

      requestAnimationFrame(async () => {
        await page.render({
          canvasContext: context,
          viewport: viewport,
        }).promise;

        this.pdfLoadedValue = true;
        this.updateControls();
        this.renderAnnotationsForCurrentPage();
        this.isRenderingValue = false;
        this.showLoadingSpinner(false);
        this.updateActiveThumbnail();
      });
    } catch (error) {
      console.error('Error rendering page:', error);
      this.isRenderingValue = false;
      this.showLoadingSpinner(false);
    }
  }

  // Annotation Management
  async loadAnnotations() {
    this.annotations = await this.annotationService.fetchAnnotations();
    this.renderAnnotationsForCurrentPage();
  }

  renderAnnotationsForCurrentPage() {
    const pageAnnotations = this.annotations.filter(a => a.pageIndex === this.indexValue - 1);
    this.annotationLayerTarget.innerHTML = '';

    pageAnnotations.forEach(annotation => {
      this.createAnnotationElement(annotation);
    });
  }

  createAnnotationElement(annotation) {
    // Ensure the canvas has an ID
    if (!this.contentTarget.id) {
      this.contentTarget.id = `pdf-canvas-${Date.now()}`;
    }

    const element = document.createElement('div');
    element.className = 'pdf-text-input';

    // Set up controller
    element.setAttribute('data-controller', 'pdf-annotation');
    element.setAttribute('data-pdf-annotation-id-value', annotation.id);
    element.setAttribute('data-pdf-annotation-page-index-value', annotation.pageIndex);
    element.setAttribute('data-pdf-annotation-font-size-value', annotation.fontSize);
    element.setAttribute('data-pdf-annotation-color-value', JSON.stringify(annotation.color));
    element.setAttribute('data-pdf-annotation-text-value', annotation.value);
    element.setAttribute('data-pdf-annotation-canvas-id-value', this.contentTarget.id);
    element.setAttribute('data-pdf-annotation-x-value', annotation.rect[0] * this.zoomValue);
    element.setAttribute('data-pdf-annotation-y-value', annotation.rect[1] * this.zoomValue);

    // Add event listeners for annotations
    element.addEventListener('annotation:updated', this.handleAnnotationUpdated.bind(this));
    element.addEventListener('annotation:deleted', this.handleAnnotationDeleted.bind(this));
    element.addEventListener('annotation:moved', this.handleAnnotationMoved.bind(this));

    // Add inner elements
    const innerText = document.createElement('div');
    innerText.setAttribute('data-pdf-annotation-target', 'input');

    const deleteButton = document.createElement('button');
    deleteButton.className = 'pdf-text-delete-button';
    deleteButton.innerHTML = '&#10005;';
    deleteButton.setAttribute('data-pdf-annotation-target', 'deleteButton');

    element.appendChild(innerText);
    element.appendChild(deleteButton);
    this.annotationLayerTarget.appendChild(element);

    return element;
  }

  addTextAnnotation(event) {
    console.log('addTextAnnotation:', event.detail);

    // Check if annotation mode is active and click is directly on the annotation layer
    if (!this.drawingMode || event.target !== this.annotationLayerTarget) {
      return;
    }

    // Calculate position from click
    const rect = this.annotationLayerTarget.getBoundingClientRect();
    console.log(event.clientX, event.clientY, rect.left, rect.top);
    const x = (event.clientX - rect.left) / this.zoomValue;
    const y = (event.clientY - rect.top) / this.zoomValue;
    console.log(x, y, x + 100, y + 30);

    // Generate a temporary ID
    const id = `temp-${Date.now()}`;

    // Create annotation data
    const newAnnotation = {
      id,
      annotationType: ANNOTATION_TYPES.FREETEXT,
      pageIndex: this.indexValue - 1,
      rect: [x, y, x + 100, y + 30], // Default size
      fontSize: parseInt(window.getComputedStyle(event.target).fontSize),
      color: [0, 0, 0],
      value: '',
    };

    this.annotations.push(newAnnotation);

    // Create and focus the element
    const textInput = this.createAnnotationElement(newAnnotation);
    const innerText = textInput.querySelector('[data-pdf-annotation-target="input"]');

    if (innerText) {
      innerText.contentEditable = true;
      innerText.focus();
    }
  }

  calculateRealPosition(rect) {
    const top = rect[0] / this.zoomValue;
    const left = rect[1] / this.zoomValue;
    const boxWidth = rect[3] - rect[1];
    const boxHeight = rect[2] - rect[0];
    const right = left + boxWidth;
    const bottom = top + boxHeight;

    return [top, left, bottom, right];
  }

  // Event Handlers
  handleAnnotationUpdated(event) {
    const { id, value, rect, fontSize, color, pageIndex } = event.detail;
    const realPosition = this.calculateRealPosition(rect);

    // Find and update the annotation
    const annotationIndex = this.annotations.findIndex(a => a.id === id);
    if (annotationIndex >= 0) {
      this.annotations[annotationIndex] = {
        ...this.annotations[annotationIndex],
        value,
        rect: realPosition,
        fontSize,
        color,
        pageIndex,
      };
    } else {
      // Add new annotation if it doesn't exist
      this.annotations.push({
        id,
        annotationType: ANNOTATION_TYPES.FREETEXT,
        pageIndex,
        rect: realPosition,
        fontSize,
        color,
        value,
      });
    }

    // Use debounced save
    this.debouncedSaveAnnotation(id, value, realPosition, fontSize, color, pageIndex);
  }

  handleAnnotationDeleted(event) {
    const { id } = event.detail;

    this.annotations = this.annotations.filter(a => a.id !== id);

    this.deleteAnnotationFromBackend(id);
  }

  handleAnnotationMoved(event) {
    const { id, rect } = event.detail;

    // Update local state
    const annotationIndex = this.annotations.findIndex(a => a.id === id);
    if (annotationIndex >= 0) {
      this.annotations[annotationIndex].rect = this.calculateRealPosition(rect);

      // Debounce position updates
      this.debouncedSaveAnnotation(
        id,
        this.annotations[annotationIndex].value,
        this.annotations[annotationIndex].rect,
        this.annotations[annotationIndex].fontSize,
        this.annotations[annotationIndex].color,
        this.annotations[annotationIndex].pageIndex
      );
    }
  }

  // Navigation Methods
  nextPage() {
    if (this.indexValue < this.totalPagesValue) {
      this.indexValue++;
      this.renderCurrentPage();
    }
  }

  previousPage() {
    if (this.indexValue > 1) {
      this.indexValue--;
      this.renderCurrentPage();
    }
  }

  goToFirstPage() {
    if (this.indexValue !== 1) {
      this.indexValue = 1;
      this.renderCurrentPage();
    }
  }

  goToLastPage() {
    if (this.indexValue !== this.totalPagesValue) {
      this.indexValue = this.totalPagesValue;
      this.renderCurrentPage();
    }
  }

  goToPage(event) {
    const page = parseInt(event.currentTarget.dataset.page);
    if (page !== this.indexValue) {
      this.indexValue = page;
      this.renderCurrentPage();
    }
  }

  indexIndicatorKeyDown(event) {
    if (event.key === 'Enter') {
      const newPage = parseInt(event.target.value);
      if (newPage && newPage >= 1 && newPage <= this.totalPagesValue && newPage !== this.indexValue) {
        this.indexValue = newPage;
        this.renderCurrentPage();
      } else {
        event.target.value = this.indexValue;
      }
    }
  }

  // Zoom Controls
  zoomIn() {
    if (!this.atMaxZoom()) {
      this.zoomValue = Math.min(this.zoomValue + ZOOM.STEP, ZOOM.MAX);
      this.renderCurrentPage();
    }
  }

  zoomOut() {
    if (!this.atMinZoom()) {
      this.zoomValue = Math.max(this.zoomValue - ZOOM.STEP, ZOOM.MIN);
      this.renderCurrentPage();
    }
  }

  atMaxZoom() {
    return this.zoomValue >= ZOOM.MAX;
  }

  atMinZoom() {
    return this.zoomValue <= ZOOM.MIN;
  }

  // UI Updates
  updateActiveThumbnail() {
    if (!this.hasThumbnailContainerTarget) return;

    this.thumbnailContainerTarget.querySelectorAll('.thumbnail').forEach((thumbnail, i) => {
      if (i + 1 === this.indexValue) {
        thumbnail.classList.add(...this.activeValue.split(' '));
      } else {
        thumbnail.classList.remove(...this.activeValue.split(' '));
      }
    });
  }

  updateControls() {
    if (this.hasIndexIndicatorTarget) {
      this.indexIndicatorTarget.value = this.indexValue;
    }

    if (this.hasZoomIndicatorTarget) {
      this.zoomIndicatorTarget.textContent = Math.round(this.zoomValue * 100) + '%';
    }

    this.disableButtonIfNeeded(this.previousPageButtonTarget, this.indexValue <= 1);
    this.disableButtonIfNeeded(this.nextPageButtonTarget, this.indexValue >= this.totalPagesValue);
    this.disableButtonIfNeeded(this.zoomInButtonTarget, this.atMaxZoom());
    this.disableButtonIfNeeded(this.zoomOutButtonTarget, this.atMinZoom());
  }

  disableButtonIfNeeded(btn, condition) {
    if (btn) {
      const isDisabled = !this.pdfLoadedValue || condition;
      btn.disabled = isDisabled;
      btn.classList.toggle('opacity-50', isDisabled);
    }
  }

  showLoadingSpinner(show) {
    if (this.hasLoadingIndicatorTarget) {
      this.loadingIndicatorTarget.classList.toggle('hidden', !show);
    }
  }

  // Annotation Mode Controls
  enableDrawing() {
    this.drawingMode = true;
    this.annotationLayerTarget.style.cursor = 'text';

    // Enable pointer events on the annotation layer
    this.annotationLayerTarget.classList.remove('pointer-events-none');
    this.annotationLayerTarget.classList.add('pointer-events-auto');

    // Toggle button visibility
    this.editButtonTarget.classList.add('hidden');
    this.cancelButtonTarget.classList.remove('hidden');
  }

  disableAnnotations() {
    this.drawingMode = false;
    this.annotationLayerTarget.style.cursor = 'default';

    // Disable pointer events on the annotation layer
    this.annotationLayerTarget.classList.remove('pointer-events-auto');
    this.annotationLayerTarget.classList.add('pointer-events-none');

    // Toggle button visibility
    this.editButtonTarget.classList.remove('hidden');
    this.cancelButtonTarget.classList.add('hidden');
  }

  // Backend Communication
  async saveAnnotationToBackend(id, value, rect, fontSize, color, pageIndex) {
    const isNewAnnotation = id.startsWith('temp-');

    const annotationData = {
      annotation: {
        page_index: pageIndex,
        content: value,
        position_data: { rect },
        style_data: { fontSize, color },
      },
    };

    const result = await this.annotationService.saveAnnotation(id, annotationData, isNewAnnotation);

    // Update ID if this was a new annotation

    if (isNewAnnotation && result.id) {
      const annotationIndex = this.annotations.findIndex(a => a.id === id);
      if (annotationIndex >= 0) {
        // Update the local state
        this.annotations[annotationIndex].id = result.id;

        // Update the DOM element
        const element = document.querySelector(`[data-pdf-annotation-id-value="${id}"]`);

        if (element) {
          element.dataset.pdfAnnotationIdValue = result.id;
        }
      }
    }
  }

  async deleteAnnotationFromBackend(id) {
    await this.annotationService.deleteAnnotation(id);
  }
}
