# frozen_string_literal: true

module Tagsets
  class InformedBadgesComponent < ViewComponent::Base
    attr_reader :tagset, :show_limit

    def initialize(tagset:, show_limit: 4)
      super
      @tagset = tagset
      @show_limit = show_limit
    end

    def informed_targets
      @informed_targets ||= InformedTarget.load_list(tagset.informed)
    end

    def informed_targets_count
      @informed_targets_count ||= informed_targets.count
    end

    def informed_targets_main
      @informed_targets_main ||= InformedTarget.load_list(tagset.informed_main)
    end

    def informed_targets_cc
      @informed_targets_cc ||= InformedTarget.load_list(tagset.informed_cc)
    end

    def informed_targets_bcc
      @informed_targets_bcc ||= InformedTarget.load_list(tagset.informed_bcc)
    end

    def informed_targets_avatars
      @informed_targets_avatars ||= begin
        avatars = informed_targets.take(show_limit).sort_by(&:full_name).map do |target|
          if target.picture_url.present?
            tag.img(
              src: target.picture_url,
              class: 'rounded-full w-6 h-6 bg-[#E6EAEC] border border-white',
              style: 'margin-right: -15px'
            )
          else
            badge_color = %w[bg-spirit-orange bg-spirit-red bg-spirit-green bg-spirit-brigthred bg-spirit-blue]
            tag.div(
              target.initials,
              class: "rounded-full w-6 h-6 bg-[#E6EAEC] border border-white text-white text-[#909399] flex justify-center items-center leading-[13px] #{badge_color.sample}",
              style: 'margin-right: -15px'
            )
          end
        end

        if avatars_overflow?
          hidden_targets_count = informed_targets_count - (show_limit - 1)

          avatars[-1] = tag.div(
            "+#{hidden_targets_count}",
            class: 'rounded-full w-6 h-6 bg-[#E6EAEC] border border-white text-[#909399] flex justify-center items-center font-bold text-[10px] leading-[13px]'
          )
        end

        avatars
      end
    end

    def names(targets_list)
      targets_list.map(&:full_name).sort.join(', ')
    end

    private

    def avatars_overflow?
      informed_targets_count > show_limit
    end

    class InformedTarget
      def self.load_list(tagset_informed_ids)
        users = ::User.where(:id.in => tagset_informed_ids).only(:first_name, :last_name, :picture_url)
        # Informed ids can be formatted as Klass#id to map generic targets
        contact_ids = tagset_informed_ids.select { _1.to_s.start_with?('Doc#') }.map { _1.split('#').last }
        contacts = ::Doc.where(:id.in => contact_ids)

        users.map { self::User.new(_1) } + contacts.map { self::Contact.new(_1) }
      end

      def initialize(target)
        @target = target
      end

      def initials
        [first_name.to_s[0], last_name.to_s[0]].join.upcase
      end

      class User < InformedTarget
        delegate :first_name, :last_name, :full_name, :picture_url, to: :@target
      end

      class Contact < InformedTarget
        def first_name
          @target.first_name.presence || @target.email.split('@').first
        end

        def last_name
          @target.last_name.presence
        end

        def full_name
          @target.full_name.presence || [first_name, last_name].join(' ')
        end

        def picture_url
          nil
        end
      end
    end
  end
end
