# frozen_string_literal: true

module SpiritHub
  class ChannelsController < ApplicationController
    before_action :set_channel, only: %i[show destroy favorite rename]
    before_action :skip_authorization, :skip_policy_scope

    def show_modal
      render ChatbotModalComponent.new(current_user, nil)
    end

    def new
      command = SpiritHub::Channels::Build.call(params, current_user, session)
      if command.success?
        @channel = command.result
        render_chatbot_modal
      else
        respond_with_error(error_message: 'Erreur lors de la création', error_status: :internal_server_error)
      end
    end

    def show
      render_chatbot_modal
    end

    def create
      command = SpiritHub::Channels::Create.call(params, current_user, session)
      @channel = command.result

      if command.success?
        render_chatbot_modal
      else
        respond_with_error(error_message: 'Erreur lors de la création', error_status: :internal_server_error)
      end
    end

    def update
      SpiritHub::Channels::Update.call(params, current_user)
      head :ok
    end

    def rename
      @channel.update(name: params[:channel][:name])
      is_favorite = current_user.favorite_channel_ids.include?(@channel.id)

      render turbo_stream: [
        turbo_stream.replace("rename_channel_#{@channel.id}",
                             ChatbotModal::ChannelItemComponent.new(channel: @channel, current_channel: @channel, is_favorite:))
      ]
    end

    def favorite
      SpiritHub::Channels::Favorite.call(params, current_user)

      render_chatbot_modal
    end

    def unfavorite
      SpiritHub::Channels::Unfavorite.call(params, current_user)

      render_chatbot_modal
    end

    def destroy
      if @channel.destroy
        @channel = nil
        render_chatbot_modal
      else
        flash[:error] = @channel.errors.full_messages.join(', ')
        render :show
      end
    end

    private

    def set_channel
      @channel = Channel.find(params[:id])
    end

    def render_chatbot_modal
      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace('chatbot-modal',
                                 ChatbotModalComponent.new(current_user, @channel))
          ]
        end

        format.html { render ChatbotModalComponent.new(current_user, @channel) }
      end
    end
  end
end
