# frozen_string_literal: true

module SpiritHub
  class DirectoriesController < ApplicationController
    before_action :find_directory
    before_action :find_models, only: %i[index sort]
    before_action :find_tagset, only: %i[add_doc_to_tagset_doc_form add_doc_to_tagset_doc remove remove_doc_from_tagset_doc duplicate]

    include Datatablable
    include StateForm

    def index
      authorize(@directory, policy_class:)

      respond_to(&:html)
    end

    def sort
      authorize(@directory, policy_class:)

      render partial: table_partial
    end

    def add_doc_to_completeness_form
      completeness = SpiritHub::Completeness.find(params[:completeness_id])
      state = SpiritHub::HubdocCycle.find_state_by_key(:classify)

      authorize(completeness.tagset, policy_class:)

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_to_completness_form', locals: { completeness:, parent_ids: @parent_ids, state: })
      ]
    end

    def add_doc_to_completeness
      completeness = SpiritHub::Completeness.find_by(id: params[:completeness_id])

      authorize(completeness.tagset, policy_class:)

      tagset_doc = Tagset.create_tagset_doc_from_file(params[:files], @user)
      tagset_doc.add_tagset_doc_to_completeness(completeness, @user)

      set_user_favorite_tagset_ids
      set_columns
      @turbo_actions << turbo_stream.replace("completeness_#{completeness.id}", partial: 'spirit_hub/directories/row',
                                                                                locals: { model: tagset_doc.reload, parent_ids: @parent_ids, show: true, show_docs: true })

      flash.now[:notice] = 'Documents ajouté'
      update_flash_message

      update_view_completion(tagset_doc)
      add_action(tagset_doc, params)

      render turbo_stream: @turbo_actions
    end

    def add_doc_to_tagset_doc_form
      authorize(@tagset, policy_class:)

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_to_tagset_doc_form', locals: { tagset: @tagset, parent_ids: @parent_ids })
      ]
    end

    def add_doc_to_tagset_doc
      authorize(@tagset, policy_class:)

      previous_doc_ids = @tagset.doc_ids.to_a.dup
      @tagset.add_doc_to_tagset_doc(params[:files], @user)
      @tagset.reload

      flash.now[:notice] = 'Documents ajouté'
      update_flash_message
      set_user_favorite_tagset_ids
      set_columns

      update_view_docs(previous_doc_ids, @tagset)
      update_view_completion(@tagset)

      render turbo_stream: @turbo_actions
    end

    def remove
      authorize(@tagset, policy_class:)

      completeness = SpiritHub::Completeness.find_by(object_ids: @tagset.id)

      SpiritHub::TagsetPurger.call(@tagset, @user)

      flash.now[:notice] = "#{@tagset.doc? ? 'Document' : 'Dossier'} supprimé"
      update_flash_message
      update_view_completion(@tagset)

      if completeness
        set_user_favorite_tagset_ids
        set_columns

        @turbo_actions << turbo_stream.replace("tagset_#{@tagset.id}", partial: 'spirit_hub/directories/completeness',
                                                                       locals: { model: completeness.reload, parent_ids: @parent_ids, show: true })
      else
        @turbo_actions << turbo_stream.remove("tagset_#{@tagset.id}")
        Array(@tagset.doc_ids).each do |doc_id|
          @turbo_actions << turbo_stream.remove("doc_#{doc_id}")
        end
      end

      render turbo_stream: @turbo_actions
    end

    def remove_doc_from_tagset_doc
      authorize(@tagset, policy_class:)

      doc = Doc.find(params[:doc_id])
      previous_doc_ids = @tagset.doc_ids.to_a.dup
      @parent_ids.pop

      @tagset.remove_doc_from_tagset_doc(doc, @user)

      flash.now[:notice] = 'Document supprimé'
      update_flash_message

      set_user_favorite_tagset_ids
      set_columns
      update_view_docs(previous_doc_ids, @tagset)
      update_view_completion(@tagset)

      render turbo_stream: @turbo_actions
    end

    def bulk_delete
      authorize(@directory, policy_class:)

      ids = params[:ids].split(',')

      Tagset.where(:id.in => ids).each do |tagset|
        tagset.current_user_id = @user.id
        tagset.destroy
      end

      flash.now[:notice] = 'Documents supprimés'

      render turbo_stream: [
        *ids.map { |id| turbo_stream.remove("tagset_#{id}") },
        turbo_stream.prepend(:flash_messages, FlashMessagesComponent.new(flash)),
        turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -ids.length })

      ]
    end

    def duplicate
      authorize(@tagset, policy_class:)

      SpiritHub::TagsetDuplicater.call(@tagset)

      flash[:notice] = 'Dossier dupliqué'
      render turbo_stream: [turbo_stream.reload]
    end

    private

    def find_models
      @operation = @directory.operation
      @journal_entries = @directory.journal_entries.order(created_at: :asc)
      scope = policy_scope(Tagset, policy_scope_class:).where(:id.in => @directory.tagset_ids.to_a)
      @models = find_models_page(scope, default_sort_by: 'cleaned_main_key')
    end

    def find_directory
      @parent_ids = params[:parent_ids]&.split(',') || []
      @directory = policy_scope(Tagset, policy_scope_class:).find_by!(id: params[:id])
    end

    def find_tagset
      @tagset = policy_scope(Tagset, policy_scope_class:).find_by(id: params[:tagset_id])
    end

    def update_view_completion(model)
      @parent_ids.each do |id|
        next unless (parent = Tagset.find_by(id:))

        @turbo_actions << turbo_stream.update("tagset_#{parent.id}_current_state_percent", partial: 'spirit_hub/common/current_state_percent',
                                                                                           locals: { model: parent.reload })
      end
      @turbo_actions << turbo_stream.update("tagset_#{@directory.id}_global_avancement", partial: 'spirit_hub/common/percent_completion',
                                                                                         locals: { percent: @directory.reload.content_completeness })

      return unless (phases = model&.operation&.phases)

      phases.each do |phase|
        @turbo_actions << turbo_stream.update("phase_percentage_#{phase.id}", partial: 'spirit_hub/common/percent_completion', locals: { percent: phase.percentage })
      end
    end

    def update_view_docs(previous_doc_ids, tagset)
      previous_doc_ids.each do |id|
        @turbo_actions << turbo_stream.remove("doc_#{id}")
      end

      @turbo_actions << turbo_stream.replace("tagset_#{tagset.id}", partial: 'spirit_hub/directories/row',
                                                                    locals: { model: tagset, parent_ids: @parent_ids, show: true, show_docs: true })
    end
  end
end
