# frozen_string_literal: true

module SpiritHub
  class ActorFormatter
    CONTACT_PREFIX = 'c_'
    GROUP_PREFIX = 'g_'
    SENTENCE_CONNECTOR = ' et '

    class << self
      def actor_ids_to_select(ids, options = {})
        actors_to_select(find_actors_from_ids(ids), options)
      end

      def actor_ids_to_json(ids, options = {})
        actors_to_json(find_actors_from_ids(ids), options)
      end

      def actor_ids_to_sentence(ids, options = {})
        actors_to_sentence(find_actors_from_ids(ids), options)
      end

      def actors_to_select(actors, options = {})
        wrap_actors(actors).map { |actor| [actor.text_value(options), actor.id_value] }
      end

      def actors_to_json(actors, options = {})
        wrap_actors(actors).map { |actor| { value: actor.id_value, text: actor.text_value(options) } }
      end

      def actors_to_sentence(actors, options = {})
        wrap_actors(actors).map { |actor| actor.text_value(options) }.compact.to_sentence(last_word_connector: SENTENCE_CONNECTOR)
      end

      def actors_from_ids(actor_ids)
        actor_ids.map do |id_value|
          actor = GroupActor.from_id(id_value) || DocActor.from_id(id_value)
          actor ||= UserActor.from_id(id_value) # fallback
          actor
        end.compact
      end

      private

      def wrap_actors(entities)
        entities.uniq.map do |entity|
          case entity
          when User then UserActor.new(entity)
          when Group then GroupActor.new(entity)
          when Doc then DocActor.new(entity)
          else
            raise ArgumentError, "Unsupported entity type: #{entity.class.name}"
          end
        end
      end

      def find_actors_from_ids(ids)
        ids = Array(ids).compact

        user_ids = find_user_ids_form_ids(ids)
        group_ids = find_group_ids_form_ids(ids)
        contact_ids = find_contact_ids_form_ids(ids)

        users = user_ids.present? ? User.where(:id.in => user_ids) : []
        groups = group_ids.present? ? Group.where(:id.in => group_ids) : []
        contacts = contact_ids.present? ? Doc.where(:id.in => contact_ids) : []

        (users + groups + contacts).compact
      end

      def find_user_ids_form_ids(ids)
        ids.reject { |id| id.to_s.include?(GROUP_PREFIX) || id.to_s.include?(CONTACT_PREFIX) }
      end

      def extract_ids_with_prefix(ids, prefix)
        ids.select { |id| id.to_s.include?(prefix) }.map { |id| id.to_s.gsub(prefix, '') }
      end

      def find_group_ids_form_ids(ids)
        extract_ids_with_prefix(ids, GROUP_PREFIX)
      end

      def find_contact_ids_form_ids(ids)
        extract_ids_with_prefix(ids, CONTACT_PREFIX)
      end
    end

    class BaseActor
      attr_reader :entity

      def initialize(entity)
        @entity = entity
      end

      def text_value(_options = {})
        raise NotImplementedError
      end

      def id_value
        raise NotImplementedError
      end
    end

    class UserActor < BaseActor
      def text_value(options = {})
        string = entity.full_name
        string += " (#{entity.email})" unless options[:without_indicator] || options[:without_email]
        string
      end

      def id_value
        entity.id.to_s
      end

      def self.from_id(id_value)
        User.find(id_value.to_s)
      end
    end

    class GroupActor < BaseActor
      PREFIX = SpiritHub::ActorFormatter::GROUP_PREFIX

      def text_value(options = {})
        string = entity.name
        string += " (#{entity.user_count} membres)" unless options[:without_indicator]
        string
      end

      def id_value
        PREFIX + entity.id.to_s
      end

      def self.from_id(id_value)
        id = id_value.to_s.gsub(PREFIX, '') if id_value.to_s.starts_with?(PREFIX)
        Group.find(id) if id
      end
    end

    class DocActor < BaseActor
      PREFIX = SpiritHub::ActorFormatter::CONTACT_PREFIX

      def text_value(options = {})
        string = "#{entity.first_name} #{entity.last_name}"
        string += " (#{entity.email})" unless options[:without_indicator] || options[:without_email]
        string
      end

      def id_value
        PREFIX + entity.id.to_s
      end

      def self.from_id(id_value)
        id = id_value.to_s.gsub(PREFIX, '') if id_value.to_s.starts_with?(PREFIX)
        Doc.where(type: 'contact').find(id) if id
      end
    end
  end
end
