# frozen_string_literal: true

module SpiritHub
  module LLM
    module Tools
      class DocsTool < ::LLM::Tools::PawTool
        slug :spirit_hub_docs_tool

        define_function( # rubocop:disable Metrics/BlockLength
          :get_results,
          description: 'DocsTool: Advanced document (Space type="document") search with semantic ' \
            'and traditional filtering capabilities.'
        ) do
          property(
            :meaning,
            type: 'string',
            description: <<~DESC,
              Semantic search using AI embeddings to find conceptually similar documents.
              Matches content based on meaning rather than exact keywords. Example: searching
              "vacation policies" finds "leave management" documents.
            DESC
            required: false
          )

          property(
            :title_contains,
            type: 'string',
            description: <<~DESC,
              Text-based search within document titles. Supports partial matches and file
              extensions (e.g., ".pdf"). Case-insensitive.
            DESC
            required: false
          )

          property(
            :type,
            type: 'string',
            description: <<~DESC,
              Filter by document type: "document" (documents), "book" (publications),
              "contact" (people/organizations), "email" (correspondence).
            DESC
            required: false
          )

          property(
            :labels,
            type: 'array',
            description: <<~DESC,
              Filter by document labels. Multiple labels use AND logic (all must match).
              Example: ["invoice", "2023"] returns only documents with both labels.
            DESC
            required: false
          ) do
            item type: 'string'
          end

          property(
            :date_start,
            type: 'string',
            description: 'Filter documents created after this date (ISO-8601: YYYY-MM-DDTHH:mm:ss.sssZ)',
            required: false
          )

          property(
            :date_end,
            type: 'string',
            description: 'Filter documents created before this date (ISO-8601: YYYY-MM-DDTHH:mm:ss.sssZ)',
            required: false
          )

          property(
            :operation_id,
            type: 'string',
            description: 'Id of the operation associated with the document being searched, in bson id format',
            required: false
          )
        end

        define_function(
          :create_doc,
          description: 'DocsTool: Create a new document with formatted content. Supports HTML formatting for book type.'
        ) do
          property(
            :format,
            type: 'string',
            description: <<~DESC,
              Document format. Use "book" for HTML-formatted content or "docx" for
              Word documents. Default: "book"
            DESC
            required: true
          )
        end

        define_function(
          :send_email,
          description: 'DocsTool: Send new email or reply to existing thread with optional attachments.'
        ) do
          property(
            :recipients,
            type: 'string',
            description: <<~DESC,
              Recipient email addresses. Format: comma-separated list of "Name <email@example.com>"
              or plain email addresses.
            DESC
            required: true
          )

          property(
            :body,
            type: 'string',
            description: <<~DESC,
              Email body in HTML format. Supports basic formatting tags for structure and
              emphasis. Do not include <head> or <style> sections.
            DESC
            required: true
          )
        end

        define_function(
          :embed_in_worlds,
          description: 'DocsTool: Index document in vector database for semantic search across knowledge domains.'
        ) do
          property(
            :id,
            type: 'string',
            description: 'id of the space type=document in bson id format, with a length of 24 characters',
            required: true
          )
          property(
            :world_slugs,
            type: 'array',
            description: <<~DESC,
              Knowledge domains for document indexing. Valid values:
              #{World.pluck(:slug).compact.to_array.join(', ')}.
              Document will be searchable within these domains.
            DESC
            required: true
          ) do
            item type: 'string'
          end
        end

        define_function(
          :get_content,
          description: 'DocsTool: Get the text content of a doc.'
        ) do
          property(
            :id,
            type: 'string',
            description: 'id of the space type=document in bson id format, with a length of 24 characters',
            required: true
          )
        end

        define_function(
          :get_infos,
          description: 'DocsTool: Get all the informations for a doc. Superfields are metadata, for a \
            email (type=email) you could find all related infos in superfields'
        ) do
          property(
            :id,
            type: 'string',
            description: 'id of the space type=document in bson id format, with a length of 24 characters',
            required: true
          )
        end

        define_function(
          :get_suggests,
          description: 'DocsTool: Get AI-powered suggestions for document (Space type="document") \
          classification and organization.'
        ) do
          property(
            :id,
            type: 'string',
            description: 'id of the space type=document in bson id format, with a length of 24 characters',
            required: true
          )
          property(
            :type,
            type: 'string',
            description: <<~DESC,
              Suggestion type: "documents" (similar content), "spaces" (recommended folders),
              or "projects" (related projects). Default: "spaces"
            DESC
            required: true
          )
        end

        define_function(
          :update_infos,
          description: 'DocsTool: Update document metadata, organization, and classification.'
        ) do
          property(
            :id,
            type: 'string',
            description: 'id of the space type=document in bson id format, with a length of 24 characters',
            required: true
          )
          property(
            :superfields,
            type: 'string',
            description: <<~DESC,
              JSON metadata object. Example: {"status": "reviewed", "priority": "high"}.
              Set value to null to remove a field.
            DESC
            required: false
          )

          property(
            :labels,
            type: 'array',
            description: "Classification tags from predefined list: #{Orgtag.all_names.to_array.join(', ')}",
            required: false
          ) do
            item type: 'string'
          end
        end

        def get_results(**properties)
          Rails.logger.tagged('LLM').debug "Tools Get Results used : #{properties[:title_contains]}"
          validate_user_presence

          search = SpiritHub::Search::SpacesSearch.new(query_from(properties), { user: })
          search.json_serialized_array
        end

        def get_suggests(id:, type: 'spaces')
          Rails.logger.tagged('LLM').debug 'Tools Get Suggests used'
          validate_user_presence

          space = Tagset.readable_by(user).find_by(id:)
          doc = Doc.find_by(id: space.doc_id)
          return 'Document not found' if doc.nil?

          doc.suggests_options(type, user:)
        end

        def embed_in_worlds(id:, world_slugs:)
          Rails.logger.debug "LLM : embed doc content #{id}"
          validate_user_presence

          space = Tagset.readable_by(user).find_by(type: 'document', id:)
          doc = Doc.find_by(id: space.doc_id)
          return 'Document not found' if doc.nil?
          return 'World slugs missing' if world_slugs.empty?

          begin
            doc.embed_in_worlds(world_slugs:)
          rescue StandardError => e
            Rails.logger.debug "LLM : embed doc content failed : #{e.message}"
            'Something went wrong and the document could not be embeded'
          end
        end

        def get_content(id:)
          Rails.logger.debug "LLM : doc get content #{id}"
          validate_user_presence

          space = Tagset.readable_by(user).find_by(type: 'document', id:)
          return 'Document not found' if space.nil?

          doc = Doc.find_by(id: space.doc_id)
          return 'File not found' if doc.nil?

          text = doc.text
          doc.extract_text if text.blank?
          doc.text
        end

        def get_infos(id:)
          Rails.logger.debug "LLM : doc get infos #{id}"
          validate_user_presence

          space = Tagset.readable_by(user).find_by(id:)
          return 'Document not found' if space.nil?

          space.channel_id = nil
          SpiritHub::SpaceSerializer.new(space, scope: user).as_json
        end

        def update_infos(**properties)
          Rails.logger.debug('LLM : doc update infos')
          validate_user_presence

          doc_id = properties.delete(:id)
          return 'Missing doc id' if doc_id.blank?

          doc = find_and_validate_doc(doc_id)
          return doc if doc.is_a?(String) # Retourne le message d'erreur si c'en est un

          update_result = safe_update_doc(doc, properties)
          return update_result if update_result.is_a?(String) # Retourne le message d'erreur

          if doc.save
            'Doc updated'
          else
            "Doc not updated, errors : #{doc.errors.full_messages.join(', ')}"
          end
        end

        private

        def validate_user_presence
          raise 'user missing' if user.nil?
        end

        def find_and_validate_doc(doc_id)
          doc = Tagset.readable_by(user).find_by(id: doc_id)
          return 'Document not found' if doc.nil?
          return 'Not allowed to change informations on this doc' unless doc.is_writers?(user.id)

          doc
        end

        def safe_update_doc(doc, properties)
          update_doc_properties(doc, properties)
          nil # Retourne nil si tout va bien
        rescue StandardError => e
          "Error : #{e}"
        end

        # Helper method to update doc properties
        def update_doc_properties(doc, properties)
          if properties[:superfields].present?
            doc.superfields = doc.superfields.merge(
              JSON.parse(properties[:superfields])
            )
          end

          doc.title = properties[:title] if properties[:title].present?
          doc.labels = properties[:labels] if properties[:labels].present?
          doc.parent_ids = properties[:parent_ids] if properties[:parent_ids].present?
          doc.project_id = properties[:project_id] if properties[:project_id].present?
        end

        def send_email(**properties)
          validate_user_presence

          validate_email_parameters(properties)
          Rails.logger.tagged('LLM').debug("Tools Send Email used : #{properties[:subject]}")

          doc = create_email_doc(properties)
          options = build_email_options(properties, doc)

          return "Can't send an email without recipients" if options[:to].blank?

          process_email_sending(doc, options)
        end

        # Helper methods for send_email
        def validate_email_parameters(properties)
          properties[:recipients].scan(/\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b/i).each do |email|
            raise ArgumentError, "Bad recipient email #{email}, bad email address" unless EmailAddress.valid?(email)
          end
        end

        def create_email_doc(properties)
          Doc.new(
            responsible: user.id,
            title: properties[:subject],
            percent: 100,
            superfields: {
              mail_to: properties[:recipients],
              mail_from: user.email
            }.stringify_keys
          )
        end

        def build_email_options(properties, _doc)
          {
            user_id: user.id.to_s,
            to: properties[:recipients],
            subject: properties[:subject],
            from: user.email,
            cc: '',
            bcc: '',
            content: properties[:body],
            dids: properties.fetch(:attachment_ids, []).to_array.map(&:to_s)
          }
        end

        def process_email_sending(doc, options)
          mail = Agent.regular_mail(doc.id.to_s, options)
          return 'Sending error' unless mail

          if doc.save
            doc.replace_file_content(mail.to_s)
            doc.type = 'email'
            doc.get_email_infos

            if doc.save && doc.check_email_sender_address(user)
              Agent.regular_mail(doc.id.to_s, options).deliver_later
              "Email created, id: #{doc.id} subject : #{doc.title}"
            else
              "Can't use this sender email or sender is not a trusted or verified account"
            end
          else
            "Errors : #{doc.errors.full_messages.join(', ')}"
          end
        end

        def create_contact(**props)
          validate_user_presence

          title = build_contact_title(props)
          doc = build_contact_doc(title, props)

          if doc.save
            "Contact created with this id #{doc.id} and this title #{doc.title}"
          else
            "Contact not created, reason : #{doc.errors}"
          end
        end

        # Helper methods for create_contact
        def build_contact_title(props)
          title = [
            props[:company_name],
            props[:salutation],
            props[:first_name],
            props[:last_name]
          ].compact.join(' ')

          title.presence || props[:email]
        end

        def build_contact_doc(title, props)
          Doc.new(
            title:,
            responsible: user.id,
            type: 'contact',
            superfields: {
              salutation: props[:salutation],
              given_name: props[:first_name],
              surname: props[:last_name],
              companyname: props[:company_name],
              # rubocop:disable Naming/VariableNumber
              email_address_1: props[:email]
              # rubocop:enable Naming/VariableNumber
            }.stringify_keys
          )
        end

        def create_doc(**properties)
          validate_user_presence

          get_doc_title(properties)
          doc = initialize_doc(properties)

          save_doc(doc, properties)
        rescue StandardError => e
          Rails.logger.tagged('LLM').debug(e.message)
          "Doc not created, reason : #{e}"
        end

        # Helper methods for create_doc
        def get_doc_title(properties)
          properties[:title] ||
            LLM::Provider.default_provider&.answer(
              "Give me a document title based on this content #{properties[:content]}"
            )
        end

        def initialize_doc(properties)
          doc = Doc.new(responsible: user.id)
          setup_doc_format(doc, properties)
          doc
        end

        def setup_doc_format(doc, properties)
          case properties[:format]
          when 'docx'
            setup_docx_format(doc, properties)
          when 'mail'
            setup_mail_format(doc, properties)
          else
            setup_default_format(doc, properties)
          end
        end

        def setup_docx_format(doc, properties)
          doc.title = "#{properties[:title]}.docx"
          doc.type = 'file'
          docx = Docx::Document.open("#{Rails.root}/app/assets/templates/base_template.docx")
          docx.paragraphs.each do |p|
            p.each_text_run { |tr| tr.substitute('content', properties[:content]) }
          end
          doc.replace_file_content(docx.stream)
        end

        def setup_mail_format(doc, properties)
          doc.type = 'mail'
          doc.body = properties[:content]
        end

        def setup_default_format(doc, properties)
          doc.title = properties[:title]
          doc.type = 'book'
        end

        def save_doc(doc, properties = {})
          if doc.save
            process_book_content(doc, properties) if doc.type == 'book'
            "Doc created with this id #{doc.id} and this title #{doc.title}"
          else
            "Doc not created, reason : #{doc.errors.full_messages.join(', ')}"
          end
        end

        def process_book_content(doc, properties)
          html = ActionController::Base.helpers.simple_format(properties[:content])
          doc.parts.create(content_type: 'text', body: html)
        end
      end
    end
  end
end
