# frozen_string_literal: true

module SpiritHub
  class TagsetDuplicater
    TAGSET_DUPLICATED_ATTRIBUTES = %w[
      is_space
      title
      readers
      writers
      superfields
      operation_id
      type
      responsible
      current_user_id
      start_date
      end_date
      phase_id
      spirit_hubdoc
    ].freeze

    COMPLETENESS_DUPLICATED_ATTRIBUTES = %w[
      title
      superfields
    ].freeze

    class << self
      def call(*)
        new(*).call
      end

      def can_duplicate?(*)
        new(*).can_duplicate?
      end
    end

    def initialize(tagset)
      @tagset = tagset
    end

    def call
      raise 'Cannot duplicate tagset' unless can_duplicate?

      duplicated_tagset = duplicate_with_children

      update_parent_tagset_ids(duplicated_tagset)
      update_existing_duplicates(duplicated_tagset)

      duplicated_tagset
    end

    def can_duplicate?
      return false if tagset.max_duplicates.blank?
      return true if tagset.max_duplicates == '*'

      tagset.existing_duplicates.length < tagset.max_duplicates.to_i
    end

    private

    attr_reader :tagset

    def duplicate_with_children
      duplicated_tagset = Tagset.create!(extract_tagset_attributes)

      tagset.completeness.each do |cps|
        SpiritHub::Completeness.create!(extract_completeness_attributes(cps, duplicated_tagset))
      end

      tagset.tagset_ids.to_a.each do |child_id|
        child_tagset = Tagset.find(child_id)
        next unless child_tagset.directory?

        duplicated_child = SpiritHub::TagsetDuplicater.new(child_tagset).send(:duplicate_with_children)
        duplicated_tagset.tagset_ids << duplicated_child.id
      end

      duplicated_tagset.save!
      duplicated_tagset
    end

    def extract_tagset_attributes
      valid_attributes = TAGSET_DUPLICATED_ATTRIBUTES & tagset.attributes.keys
      tagset.attributes.slice(*valid_attributes).merge(tagset_ids: [])
    end

    def extract_completeness_attributes(cps, duplicated_tagset)
      valid_attributes = COMPLETENESS_DUPLICATED_ATTRIBUTES & cps.attributes.keys
      cps.attributes.slice(*valid_attributes).merge(tagset_id: duplicated_tagset.id)
    end

    def update_existing_duplicates(duplicated_tagset)
      tagset.update(existing_duplicates: tagset.existing_duplicates.to_a + [duplicated_tagset.id.to_s])
    end

    def update_parent_tagset_ids(duplicated_tagset)
      return unless (parent = tagset.parent)

      parent.tagset_ids.to_a << duplicated_tagset.id
      parent.save!
    end
  end
end
