# frozen_string_literal: true

module SpiritHub
  # Shares a tagset doc with multiple users.
  # This module is also used for "Respond" action
  class TagsetShare
    include SpiritHub::Engine.routes.url_helpers

    CONTACT_PREFIX = 'c_'
    GROUP_PREFIX = 'g_'

    class << self
      def call(...)
        new(...).call
      end

      def share_multiple_docs_from_same_tagset_individually(options)
        options.fetch(:docs).each do |doc_id, doc_sharing_options|
          call(
            **options.symbolize_keys.slice(:tagset, :current_user, :subject, :message),
            doc_ids: [doc_id],
            to: doc_sharing_options.symbolize_keys.fetch(:to)
          )
        end
      end
    end

    def initialize(options)
      options.symbolize_keys!

      @tagset = options.fetch(:tagset)
      @doc_ids = options[:doc_ids]
      @to = options.fetch(:to).map(&:presence).compact
      @cc = options.fetch(:cc, []).map(&:presence).compact
      @bcc = options.fetch(:bcc, []).map(&:presence).compact
      @subject = options.fetch(:subject)
      @message = options.fetch(:message)
      @attachments = options[:attachments] || []
      @attached_tagset_ids = options[:attached_tagset_ids] || []
      @current_user = options.fetch(:current_user)
      @action = options[:action].presence || :share
    end

    def call
      persist_attached_docs
      persist_new_contacts
      create_upload

      SpiritHub::HubdocCycle.change_state(tagset, action, current_user)

      update_informed

      create_and_deliver_email
      create_comment
      log_action

      true
    end

    private

    attr_reader(
      :tagset,
      :doc_ids,
      :to,
      :cc,
      :bcc,
      :subject,
      :message,
      :attachments,
      :current_user,
      :action,
      :upload
    )

    def persist_attached_docs
      attached_docs_as_tagsets = Tagset.create_tagset_docs_from_files(
        attachments,
        current_user,
        parents: tagset.parents,
        state: :done
      )

      attachment_tagsets.concat(attached_docs_as_tagsets)
    end

    def attachment_tagsets
      @attachment_tagsets ||= Tagset.where(type: 'document', :id.in => @attached_tagset_ids).to_a if @attached_tagset_ids.present?
      @attachment_tagsets ||= []
    end

    def persist_new_contacts
      create_new_contacts_from = proc do |target_ids|
        target_ids.uniq.map { |target_id| EmailAddress.valid?(target_id) ? find_or_create_contact(target_id) : target_id }
      end

      @to = create_new_contacts_from.call(to)
      @cc = create_new_contacts_from.call(cc)
      @bcc = create_new_contacts_from.call(bcc)
    end

    def find_or_create_contact(email_address)
      contact = Doc.contacts_with_emails([email_address]).first

      if contact.nil?
        contact = Doc.create!(
          type: 'contact',
          title: email_address,
          superfields: { 'email_address_1' => email_address }
        )
        contact.current_user_id = current_user.id
        contact.save!
      end

      CONTACT_PREFIX + contact.id.to_s
    end

    def create_comment
      return if @message.blank?

      SpiritHub::Comments::Create.call(comment_params, current_user)
    end

    def comment_params
      ActionController::Parameters.new(
        instant_message: {
          channel_key: "upload_comments_#{@upload.id}",
          text: @message
        }
      )
    end

    def create_upload
      @upload = Upload.create(doc_ids: shared_doc_ids, user_id: current_user.id)

      return unless action == :share

      SpiritHub::JournalEntry.share_action_log(
        'SHARE_LINK_CREATED',
        tagset:,
        user: current_user,
        upload_id: @upload.id,
        doc_ids: @upload.doc_ids,
        link: spirit_hub_upload_url(host: APP_CONFIG['app_host'], id: @upload.id)
      )
    end

    def shared_doc_ids
      @shared_doc_ids ||= begin
        ids = tagset.find_docs_or_children_docs.map(&:id).map(&:to_s)
        ids &= doc_ids.map(&:to_s) if doc_ids.present?
        ids += attachment_tagsets.map(&:doc_ids).flatten
        ids
      end
    end

    def update_informed
      return unless action == :share

      tagsets = tagset.directory? ? tagset.children_tagsets.where(type: 'document') : [tagset]

      tagsets.each do |t|
        t.update!(
          informed_main: new_informed_main_list,
          informed_cc: new_informed_cc_list,
          informed_bcc: new_informed_bcc_list
        )
      end
    end

    def create_and_deliver_email
      return unless to_emails.present?

      email = SpiritHub::TagsetsMailer.share(
        upload: upload.id.to_s,
        subject:,
        message:,
        to: to_emails,
        cc: cc_emails,
        bcc: bcc_emails
      )

      email.deliver_later

      create_doc_email_record(email)
    end

    def create_doc_email_record(email)
      Doc.where(type: 'email').create!(
        user: current_user,
        title: email.subject,
        body: email.body,
        superfields: {
          'mail_from' => email.from.join(','),
          'mail_to' => email.to.join(','),
          'mail_cc' => email.cc.join(','),
          'mail_bcc' => email.bcc.join(',')
        }
      )
    end

    def to_emails
      users_main.map(&:email) + contacts_main.map(&:title)
    end

    def cc_emails
      users_cc.map(&:email) + contacts_cc.map(&:title)
    end

    def bcc_emails
      users_bcc.map(&:email) + contacts_bcc.map(&:title)
    end

    def users_main
      @users_main ||= User.where(:id.in => expand_user_ids_from(to))
    end

    def users_cc
      @users_cc ||= User.where(:id.in => expand_user_ids_from(cc))
    end

    def users_bcc
      @users_bcc ||= User.where(:id.in => expand_user_ids_from(bcc))
    end

    def expand_user_ids_from(target_ids)
      target_ids.to_a.flat_map do |target_id|
        if target_id.starts_with?(GROUP_PREFIX)
          group_id = target_id.remove(GROUP_PREFIX)
          Group.find(group_id).users.pluck(:id)
        else
          target_id
        end
      end
    end

    def contacts_main
      @contacts_main ||= existing_contacts_in(to)
    end

    def contacts_cc
      @contacts_cc ||= existing_contacts_in(cc)
    end

    def contacts_bcc
      @contacts_bcc ||= existing_contacts_in(bcc)
    end

    def existing_contacts_in(target_ids)
      Doc.where(type: 'contact', :id.in => expand_contact_ids_from(target_ids))
    end

    def expand_contact_ids_from(target_ids)
      target_ids
        .select { _1.starts_with?(CONTACT_PREFIX) && !_1.include?('@') }
        .map { _1.remove(CONTACT_PREFIX) }
    end

    def new_informed_main_list
      new_ids = users_main.map(&:id) + contacts_main.map { |c| [c.class, '#', c.id].join }
      (Array(tagset.informed_main) + new_ids).uniq
    end

    def new_informed_cc_list
      new_ids = users_cc.map(&:id) + contacts_cc.map { |c| [c.class, '#', c.id].join }
      (Array(tagset.informed_cc) + new_ids).uniq
    end

    def new_informed_bcc_list
      new_ids = users_bcc.map(&:id) + contacts_bcc.map { |c| [c.class, '#', c.id].join }
      (Array(tagset.informed_bcc) + new_ids).uniq
    end

    def log_action
      case action
      when :share
        recipients = (to + cc + bcc).map(&:presence).compact
        log_code = tagset.multiple_doc? ? 'MULTIPLE_DOC_SHARED' : 'DOC_SHARED'
        log_opts = { user: current_user, tagset:, doc_ids: upload.doc_ids, recipients:, to:, cc:, bcc: }

        SpiritHub::JournalEntry.share_action_log(log_code, log_opts)
      when :respond
        SpiritHub::JournalEntry.process_action_log('DOC_RESPONDED', tagset:, user: current_user)
      end
    end
  end
end
