# frozen_string_literal: true

module SpiritHub
  class TagsetSplitter
    attr_reader(:tagset, :per_page, :user)

    class << self
      def call(...)
        new(...).call
      end
    end

    def initialize(options)
      raise ArgumentError, 'tagset is required' unless options[:tagset]
      raise ArgumentError, 'user is required' unless options[:user]

      @tagset = options[:tagset]
      raise StandardError, 'tagset must have only one doc' unless tagset.one_doc?
      raise StandardError, 'tagset doc must be a PDF' unless tagset.docs.first.pdf?

      @per_page = options[:per_page]&.to_i || 1
      @user = options[:user]
    end

    def call
      files = split_doc_into_files
      new_tagset_doc = Tagset.create_tagset_doc_from_file(files, user, title: "#{tagset.title} (découpé)", parents: tagset.parents)
      new_tagset_doc.save

      add_favorite(new_tagset_doc) if user.favorite_tagset_ids.include?(tagset.id)

      log_action
      delete_files(files)

      new_tagset_doc
    end

    private

    def split_doc_into_files
      basename = File.basename(tagset.title, '.*')
      pdf = CombinePDF.parse(tagset.docs.first.file.data)
      files = []

      pdf.pages.to_a.each_slice(per_page).with_index do |pages, index|
        new_pdf = CombinePDF.new
        pages.each { |page| new_pdf << page }

        file_path = Rails.root.join('tmp', "#{basename} [#{index + 1}].pdf")
        new_pdf.save(file_path)
        files << file_path.to_s
      end

      files
    end

    def delete_files(files)
      files.each { |file| File.delete(file) }
    end

    def add_favorite(new_tagset_doc)
      Favorite.create(user:, tagset: new_tagset_doc)
    end

    def log_action
      SpiritHub::JournalEntry.create_action_log('DOC_SPLITTED', tagset:, user:)
    end
  end
end
