#!/bin/bash
set -e

# Define core directory path
CORE_DIR="../core"
SPIRIT_HUB_DIR="."

# Verify core directory exists
if [ ! -f "$CORE_DIR/Gemfile" ]; then
  echo "❌ Could not locate Gemfile in $CORE_DIR"
  echo "Please ensure you're running this from the spirit_hub directory with core as a sibling directory"
  exit 1
fi

DOCKER_COMPOSE_FILE="docker-compose.cypress.yml"
APP_URL="http://localhost:3001"
APP_CONTAINER="cypress-core-app"

cleanup() {
  echo "🛑 Shutdown detected, removing containers..."
  docker compose -f $DOCKER_COMPOSE_FILE down
  exit 0
}

trap cleanup SIGINT SIGTERM EXIT

echo "🚀 Starting containers for Cypress tests..."
docker compose -f $DOCKER_COMPOSE_FILE up -d --remove-orphans

if [ -f "$CORE_DIR/tmp/pids/server.cypress.pid" ]; then
  echo "🔄 Cleaning up old pids..."
  rm "$CORE_DIR/tmp/pids/server.cypress.pid"
fi

wait_for_service() {
  echo "⏳ Waiting for application to be ready..."
  local retries=80
  local count=0

  while ! curl -s "$APP_URL" > /dev/null; do
    count=$((count + 1))
    if [ $count -ge $retries ]; then
      echo "❌ Rails service did not start after $retries attempts."
      echo "📝 Container logs:"
      docker logs $APP_CONTAINER
      exit 1
    fi
    echo "🔄 Checking... ($count/$retries)"
    sleep 5
  done

  echo "✅ Application is ready!"
}

# Show logs in real-time while waiting
docker logs -f $APP_CONTAINER &
LOGS_PID=$!

wait_for_service

# Kill the logs process once the service is ready
kill $LOGS_PID 2>/dev/null || true

# Before starting Cypress, ensure it's installed
ensure_cypress_installed() {
  echo "🔍 Checking Cypress installation..."
  if ! yarn cypress verify &>/dev/null; then
    echo "📦 Installing Cypress..."
    yarn cypress install
  fi
}

echo "🧪 Starting Cypress with: yarn cypress $@"
ensure_cypress_installed
CYPRESS_baseUrl="$APP_URL" yarn cypress "$@" --project ./e2e

wait
