# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::ActorFormatter, type: :formatter) do
  RSpec.shared_context 'shared actors setup' do |type|
    let!(:johndoe) { create(:user, first_name: 'John', last_name: 'Doe', email: 'johndoe@test.com') }
    let!(:janedoe) { create(:user, first_name: 'Jane', last_name: 'Doe', email: 'janedoe@test.com') }
    let!(:group1) { create(:group, name: 'Groupe 1', user_ids: [johndoe.id, janedoe.id]) }
    let!(:group2) { create(:group, name: 'Groupe 2', user_ids: []) }
    let!(:contact1) { create(:contact, first_name: 'Contact', last_name: '1', email: 'contact1@test.com') }
    let(:default_actor_ids) { [johndoe.id, janedoe.id, "g_#{group1.id}", "g_#{group2.id}", "c_#{contact1.id}"] }
    let(:default_actors) { [johndoe, janedoe, group1, group2, contact1] }
    let(:input) { type == :ids ? default_actor_ids : default_actors }
    let!(:options) { {} }
  end

  shared_examples 'for select tests' do |method, type|
    include_context 'shared actors setup', type

    subject(:value) { described_class.public_send(method, input, options) }

    it 'default' do
      expect(value.first).to eq(['John Doe (johndoe@test.com)', johndoe.id.to_s])
      expect(value[2]).to eq(['Groupe 1 (2 membres)', "g_#{group1.id}"])
      expect(value.last).to eq(['Contact 1 (contact1@test.com)', "c_#{contact1.id}"])
    end

    context 'without_indicator' do
      let!(:options) { { without_indicator: true } }

      it 'works' do
        expect(value.first).to eq(['John Doe', johndoe.id.to_s])
        expect(value[2]).to eq(['Groupe 1', "g_#{group1.id}"])
        expect(value.last).to eq(['Contact 1', "c_#{contact1.id}"])
      end
    end

    context 'without_email' do
      let!(:options) { { without_email: true } }

      it 'works' do
        expect(value.first).to eq(['John Doe', johndoe.id.to_s])
        expect(value[2]).to eq(['Groupe 1 (2 membres)', "g_#{group1.id}"])
        expect(value.last).to eq(['Contact 1', "c_#{contact1.id}"])
      end
    end

    context 'without input' do
      let!(:input) { [] }

      it 'works' do
        expect(value).to eq([])
      end
    end
  end

  describe 'actor_ids_to_select' do
    it_behaves_like 'for select tests', :actor_ids_to_select, :ids
  end

  describe 'actors_to_select' do
    it_behaves_like 'for select tests', :actors_to_select, :actors

    context 'when there are duplicates on input' do
      subject(:values) { described_class.actors_to_select(actors_with_duplicates) }

      include_context 'shared actors setup', :actors

      let(:actors_with_duplicates) { [johndoe, janedoe, group1, group2, contact1, johndoe, group1, contact1] }

      it 'does not contain duplicates' do
        value_ids = values.map(&:second)
        value_ids_uniq = value_ids.uniq

        expect(value_ids).to match_array value_ids_uniq
      end
    end
  end

  shared_examples 'for json tests' do |method, type|
    include_context 'shared actors setup', type

    subject(:value) { described_class.public_send(method, input, options) }

    it 'default' do
      expect(value.first).to eq(value: johndoe.id.to_s, text: 'John Doe (johndoe@test.com)')
      expect(value[2]).to eq(value: "g_#{group1.id}", text: 'Groupe 1 (2 membres)')
      expect(value.last).to eq(value: "c_#{contact1.id}", text: 'Contact 1 (contact1@test.com)')
    end

    context 'without_indicator' do
      let!(:options) { { without_indicator: true } }

      it 'works' do
        expect(value.first).to eq(value: johndoe.id.to_s, text: 'John Doe')
        expect(value[2]).to eq(value: "g_#{group1.id}", text: 'Groupe 1')
        expect(value.last).to eq(value: "c_#{contact1.id}", text: 'Contact 1')
      end
    end

    context 'without_email' do
      let!(:options) { { without_email: true } }

      it 'works' do
        expect(value.first).to eq(value: johndoe.id.to_s, text: 'John Doe')
        expect(value[2]).to eq(value: "g_#{group1.id}", text: 'Groupe 1 (2 membres)')
        expect(value.last).to eq(value: "c_#{contact1.id}", text: 'Contact 1')
      end
    end

    context 'without input' do
      let!(:input) { [] }

      it 'works' do
        expect(value).to eq([])
      end
    end
  end

  describe 'actor_ids_to_json' do
    it_behaves_like 'for json tests', :actor_ids_to_json, :ids
  end

  describe 'actors_to_json' do
    it_behaves_like 'for json tests', :actors_to_json, :actors
  end

  shared_examples 'for phrase tests' do |method, type|
    include_context 'shared actors setup', type

    subject(:value) { described_class.public_send(method, input, options) }

    it 'default' do
      expect(value).to eq('John Doe (johndoe@test.com), Jane Doe (janedoe@test.com), Groupe 1 (2 membres), Groupe 2 (0 membres) et Contact 1 (contact1@test.com)')
    end

    context 'without_indicator' do
      let!(:options) { { without_indicator: true } }

      it 'works' do
        expect(value).to eq('John Doe, Jane Doe, Groupe 1, Groupe 2 et Contact 1')
      end
    end

    context 'without_email' do
      let!(:options) { { without_email: true } }

      it 'works' do
        expect(value).to eq('John Doe, Jane Doe, Groupe 1 (2 membres), Groupe 2 (0 membres) et Contact 1')
      end
    end

    context 'without input' do
      let!(:input) { [] }

      it 'works' do
        expect(value).to eq('')
      end
    end
  end

  describe 'actor_ids_to_sentence' do
    it_behaves_like 'for phrase tests', :actor_ids_to_sentence, :ids
  end

  describe 'actors_to_sentence' do
    it_behaves_like 'for phrase tests', :actors_to_sentence, :actors
  end

  describe 'actors_from_ids' do
    subject { described_class.actors_from_ids(id_values) }

    let(:id_values) { [] }

    it { is_expected.to eq [] }

    context 'when ids from users, contacts and groups' do
      let(:id_values) do
        [user.id.to_s, "g_#{group.id}", "c_#{contact.id}", 'g_unexsiting-group', 'c_unexsiting-contact', 'unexisting-user']
      end

      let(:user) { create(:user) }
      let(:group) { create(:group) }
      let(:contact) { create(:contact) }

      it { is_expected.to match_array [user, group, contact] }
    end

    context 'when ids are matched wrongly (edge case)' do
      let(:id_values) do
        ["g_#{user.id}", "c_#{user.id}", 'unexisting-user']
      end

      let(:user) { create(:user) }

      it { is_expected.to match_array [] }
    end
  end
end
