# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::Docs::EmailsExtracterJob, type: :job do
  subject { described_class.new.perform(doc.id.to_s) }

  let(:doc) { create(:doc) }
  let(:doc_text) do
    <<~TEXT
      Lorem ipsum dolor sit amet, consectetur adipiscing elit, john@gmail.com,
      sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.

        jack@gmail.com

      Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
      ex ea commodo consequat.
    TEXT
  end

  before { allow(Doc).to receive(:find).with(doc.id.to_s).and_return(doc) if doc.persisted? }
  before { allow(doc).to receive(:pdf?).and_return(true) }
  before { allow(doc).to receive(:text).and_return(doc_text) }

  it 'extracts the email addresses from the content and stores them' do
    expect(doc.extracted_emails).to be_nil
    expect(doc.extracted_emails_at).to be_nil

    subject
    expect(doc.reload.extracted_emails).to match_array ['john@gmail.com', 'jack@gmail.com']
    expect(doc.extracted_emails_at).to be_present
  end

  context 'when there is an email in the content that is not from a valid domain' do
    let(:doc_text) do
      <<~TEXT
        Lorem ipsum dolor sit amet, consectetur adipiscing elit, john@gmail.com,
        sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.

          email_from@a_fake.domain

        Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
        ex ea commodo consequat.
      TEXT
    end

    it 'ignores it' do
      expect(doc.extracted_emails).to be_nil
      subject
      expect(doc.reload.extracted_emails).to match_array ['john@gmail.com']
    end
  end

  context 'when doc cannot be found' do
    let(:doc) { build(:doc) }

    it 'does not raise error' do
      expect { subject }.to_not raise_error
    end
  end

  context 'when doc is not a PDF' do
    before { allow(doc).to receive(:pdf?).and_return(false) }

    it 'does not set the fields' do
      subject

      expect(doc.reload.extracted_emails).to be_nil
      expect(doc.reload.extracted_emails_at).to be_nil
    end
  end
end
