# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::Docs::RequestSignatureJob, type: :job do
  subject { described_class.new.perform(doc_ids, recipient_ids, current_user.id) }

  let(:doc_ids) { [doc.id] }
  let(:doc) { create(:doc) }
  let(:recipient_ids) { [user1.id, user2.id] }
  let(:user1) { create(:user) }
  let(:user2) { create(:user) }
  let(:current_user) { create(:user) }

  let(:docusign_envelope_service_mock) { instance_double(::Signature::Docusign::EnvelopeService) }

  before do
    allow(::Signature::Docusign::EnvelopeService).to receive(:initialize_with_valid_credentials).and_return(
      docusign_envelope_service_mock
    )

    allow(docusign_envelope_service_mock).to receive(:create_and_send_docusign_envelope)
  end

  it 'creates a new Docusign::Envelope record with the correct data' do
    expect { subject }.to change(Signature::Docusign::Envelope, :count).by(1)

    envelope = Signature::Docusign::Envelope.last
    expect(envelope.creator).to eq current_user
    expect(envelope.docs.to_a).to match_array [doc]
    expect(envelope.signers.map(&:user).to_a).to match_array [user1, user2]
  end

  it 'sends the Docusign envelope' do
    subject

    envelope = Signature::Docusign::Envelope.last
    expect(docusign_envelope_service_mock).to have_received(:create_and_send_docusign_envelope).with(envelope)
  end

  context 'when an error is raised while generating the Docusign credentials' do
    before do
      allow(::Signature::Docusign::EnvelopeService).to receive(:initialize_with_valid_credentials).and_raise('Something went wrong')
    end

    it 'does not store the envelope record in the Db' do
      expect { subject }.to raise_error 'Something went wrong'

      expect(Signature::Docusign::Envelope.count).to eq 0
    end
  end

  context 'when an error is raised while sending the document to Docusign' do
    before do
      allow(docusign_envelope_service_mock).to receive(:create_and_send_docusign_envelope).and_raise('Something went wrong')
    end

    it 'does not store the envelope record in the Db' do
      expect { subject }.to raise_error 'Something went wrong'

      expect(Signature::Docusign::Envelope.count).to eq 0
    end
  end
end
