# frozen_string_literal: true

require 'rails_helper'

RSpec.describe Channel, type: :model do
  let(:user) { create(:user) }
  let!(:channel) { create(:channel, responsible: user.id) }

  describe 'scopes' do
    let!(:created_at_today) { create(:channel, created_at: 1.hour.ago) }
    let!(:created_at_yesterday) { create(:channel, created_at: 1.day.ago) }
    let!(:created_2_days_ago) { create(:channel, created_at: 2.days.ago) }
    let!(:created_5_days_ago) { create(:channel, created_at: 5.days.ago) }
    let!(:created_10_days_ago) { create(:channel, created_at: 10.days.ago) }

    it 'returns unarchived channels' do
      archived_channel = create(:channel, archived: true)
      expect(Channel.unarchived).not_to include(archived_channel)
    end

    it 'returns user channels' do
      expect(Channel.user_channels(user)).to include(channel)
    end

    it 'returns channels ordered by latest' do
      expect(Channel.latest.to_a).to eq([channel, created_at_today, created_at_yesterday, created_2_days_ago, created_5_days_ago, created_10_days_ago])
    end

    it 'returns channels created today' do
      expect(Channel.created_today.to_a).to eq([channel, created_at_today])
    end

    it 'returns channels created yesterday' do
      expect(Channel.created_yesterday.to_a).to eq([created_at_yesterday])
    end

    it 'returns channels created last week' do
      expect(Channel.created_last_week.to_a).to eq([created_5_days_ago])
    end

    it 'returns channels created last month' do
      expect(Channel.created_last_month.to_a).to eq([created_10_days_ago])
    end

    it 'returns channels created older than last month' do
      expect(Channel.created_older_than_last_month.count).to eq(0)
    end

    it 'returns not favorite channels for a user' do
      allow(user).to receive(:favorite_channel_ids).and_return([channel.id])
      expect(Channel.not_favorite(user)).not_to include(channel)
    end
  end

  describe 'callbacks' do
    describe '#update_ui' do
      it 'calls #broadcast_rename when name changes' do
        expect(channel).to receive(:broadcast_rename)
        channel.update(name: 'New Name')
      end

      it 'calls #broadcast_waiting_for_bot when waiting_for_bot changes' do
        expect(channel).to receive(:broadcast_waiting_for_bot)
        channel.update(waiting_for_bot: true)
      end
    end
  end

  describe 'private methods' do
    describe '#broadcast_rename' do
      it 'broadcasts the renamed channel to Turbo Streams' do
        expect(Turbo::StreamsChannel).to receive(:broadcast_replace_later_to)
          .with("channel_name_#{channel.id}",
                target: "channel_name_#{channel.id}",
                html: kind_of(String))
        channel.update(name: 'Updated Name')
      end
    end

    describe '#broadcast_waiting_for_bot' do
      it 'broadcasts waiting for bot status to Turbo Streams' do
        expect(channel).to receive(:broadcast_append_to)
          .with("channel_waiting_for_bot_#{channel.id}",
                target: "channel_waiting_for_bot_#{channel.id}",
                renderable: kind_of(SpinnerComponent))
        channel.update(waiting_for_bot: true)
      end
    end
  end
end
