# frozen_string_literal: true

require 'rails_helper'

RSpec.describe InstantMessage do
  let(:channel) { create(:channel) }
  let(:user) { create(:user) }

  describe 'callbacks' do
    it 'triggers broadcast and update callbacks after create' do
      expect_any_instance_of(described_class).to receive(:broadcast_append_to_channel)
      expect_any_instance_of(described_class).to receive(:update_waiting_for_bot)
      create(:instant_message, channel:)
    end
  end

  describe '#update_waiting_for_bot' do
    let(:channel) { create(:channel, waiting_for_bot: true) }

    it 'updates channel waiting status to false' do
      message = build(:instant_message, channel:)
      message.update_waiting_for_bot
      expect(channel.reload.waiting_for_bot).to be false
    end
  end

  describe '#broadcast_append_to_channel' do
    context 'with message type "m"' do
      let(:message) { build(:instant_message, type: 'm', channel:) }

      it 'triggers message append and system message clear' do
        expect(message).to receive(:broadcast_append_to_messages)
        expect(message).to receive(:broadcast_system_message_clear)
        message.broadcast_append_to_channel
      end
    end

    context 'with other message types' do
      let(:message) { build(:instant_message, type: 's', channel:) }

      it 'triggers system message replacement' do
        expect(message).to receive(:broadcast_replace_system_message)
        message.broadcast_append_to_channel
      end
    end
  end

  describe 'user type checks' do
    let(:bot) { create(:bot) }

    context 'with regular user' do
      let(:message) { build(:instant_message, user:) }

      it '#user? returns true' do
        expect(message.send(:user?)).to be true
      end

      it '#assistant? returns false' do
        expect(message.send(:assistant?)).to be false
      end
    end

    context 'with bot user' do
      let(:message) { build(:instant_message, user: bot) }

      it '#user? returns false' do
        expect(message.send(:user?)).to be false
      end

      it '#assistant? returns true' do
        expect(message.send(:assistant?)).to be true
      end
    end
  end

  describe '#suppressed_turbo_broadcasts?' do
    it 'always returns false' do
      message = build(:instant_message)
      expect(message.send(:suppressed_turbo_broadcasts?)).to be false
    end
  end
end
