# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::LLM::Tools::DocsTool, type: :model do
  let(:tool) do
    tool = described_class.new
    tool.user = user
    tool
  end

  before { SpiritHub::HubdocCycle.import_cycle_from_json }

  let(:user) { create(:user, :confirmed) }

  describe '#get_results' do
    subject { tool.get_results(**properties) }

    let(:properties) do
      {
        labels:,
        date_start: created_at_from,
        date_end: created_at_to,
        page:
      }
    end

    let(:labels) { %w[invoice archived] }
    let(:created_at_from) { '2021-01-01' }
    let(:created_at_to) { '2021-02-01' }
    let(:page) { 2 }

    it 'builds the docs search correctly' do
      expect(Search::SpacesSearch).to receive(:new).with(
        {
          labels:,
          date_start: created_at_from,
          date_end: created_at_to,
          page:
        },
        {
          user:
        }
      ).and_call_original

      subject
    end

    context 'when user is not set' do
      subject do
        tool = described_class.new
        tool.get_results(**properties)
      end

      it 'raises an error' do
        expect { subject }.to raise_error('user missing')
      end
    end
  end

  describe '#get_content' do
    subject { tool.get_content(id: space.id.to_s) }

    let!(:doc) { create(:doc, responsible: user.id) }
    let!(:space) { create(:tagset_doc, doc_ids: [doc.id], responsible: user.id) }

    before do
      allow_any_instance_of(Doc).to receive(:text).and_return(Faker::Lorem.sentence(word_count: 10))
    end

    it 'returns the text content from the target doc' do
      expect(doc.text).to be_present
      expect(subject).to eq doc.text
    end

    it 'returns "Document not found" when the doc is not found' do
      expect(tool.get_content(id: 123)).to eq 'Document not found'
    end
  end

  describe '#update_infos' do
    subject { tool.update_infos(id: document.id.to_s, title:, superfields: superfields.to_json) }

    let!(:doc) { create(:doc, responsible: user.id) }
    let!(:document) { create(:tagset_doc, doc_ids: [doc.id], responsible: user.id) }
    let(:title) { 'test.txt' }
    let(:superfields) { { "invoice_number": 123 } }

    it 'change title of doc' do
      expect(subject).to eq 'Doc updated'
      expect(document.reload.title).to eq title
      expect(document.reload.superfields['invoice_number']).to eq superfields[:invoice_number]
    end
  end
end
