# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(User, type: :model) do
  it 'has a valid factory' do
    expect(true).to be_truthy
  end

  let(:user) { create(:user) }
  let(:other_user) { create(:user) }

  describe '#favorite_channel_ids' do
    let!(:favorite1) { create(:favorite, user:, channel_id: 'channel_1') }
    let!(:favorite2) { create(:favorite, user:, channel_id: 'channel_2') }
    let!(:nil_channel_favorite) { create(:favorite, user:, channel_id: nil) }
    let!(:other_user_favorite) { create(:favorite, user: other_user, channel_id: 'channel_3') }

    it 'returns distinct channel IDs for user favorites' do
      expect(user.favorite_channel_ids).to contain_exactly('channel_1', 'channel_2')
    end

    it 'excludes favorites with nil channel_id' do
      expect(user.favorite_channel_ids).not_to include(nil)
    end

    it 'excludes other users favorites' do
      expect(user.favorite_channel_ids).not_to include('channel_3')
    end

    context 'when no favorites exist' do
      let(:other_user2) { create(:user) }

      it 'returns empty array' do
        expect(other_user2.favorite_channel_ids).to eq([])
      end
    end
  end

  describe '#favorite_channels' do
    let!(:channel1) { create(:channel) }
    let!(:channel2) { create(:channel) }
    let!(:favorite1) { create(:favorite, user:, channel: channel1) }
    let!(:favorite2) { create(:favorite, user:, channel: channel2) }
    let!(:other_channel) { create(:channel) }

    it 'returns channels associated with user favorites' do
      expect(user.favorite_channels).to include(channel1, channel2)
    end

    it 'excludes non-favorited channels' do
      expect(user.favorite_channels).not_to include(other_channel)
    end

    context 'when no favorite channels exist' do
      let(:other_user) { create(:user) }

      it 'returns empty relation' do
        expect(other_user.favorite_channels).to eq([])
      end
    end
  end

  describe '#admin?' do
    context 'when user has Administrator role' do
      let(:user) { create(:user, roles: ['Administrator']) }

      it { expect(user).to be_admin }
    end

    context 'when user has Superadmin role' do
      let(:user) { create(:user, roles: ['Superadmin']) }

      it { expect(user).to be_admin }
    end

    context 'when user has both admin roles' do
      let(:user) { create(:user, roles: %w[Administrator Superadmin]) }

      it { expect(user).to be_admin }
    end

    context 'when user has non-admin roles' do
      let(:user) { create(:user, roles: %w[Editor Viewer]) }

      it { expect(user).not_to be_admin }
    end

    context 'when user has no roles' do
      let(:user) { create(:user, roles: []) }

      it { expect(user).not_to be_admin }
    end

    context 'when roles contain partial matches' do
      let(:user) { create(:user, roles: ['Admin', 'Super Administrator']) }

      it { expect(user).not_to be_admin }
    end
  end
end
