# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::DirectoriesPolicy do
  let(:admin) { create(:user, roles: ['Administrator']) }
  let(:responsible) { create(:user) }
  let(:reader) { create(:user) }
  let(:writer) { create(:user) }
  let(:accounted_user) { create(:user) }
  let(:other_user) { create(:user) }
  let(:directory) { create(:tagset_directory, responsible: responsible.id, readers: [reader.id], writers: [writer.id], accounted: [accounted_user.id.to_s]) }

  subject { described_class.new(user, directory) }

  context 'when user is an admin' do
    let(:user) { admin }

    it 'permits all actions' do
      expect(subject.index?).to be true
      expect(subject.sort?).to be true
      expect(subject.add_doc_to_completeness_form?).to be true
      expect(subject.add_doc_to_completeness?).to be true
      expect(subject.add_doc_to_tagset_doc_form?).to be true
      expect(subject.add_doc_to_tagset_doc?).to be true
      expect(subject.remove?).to be true
      expect(subject.remove_doc_from_tagset_doc?).to be true
      expect(subject.bulk_delete?).to be true
      expect(subject.duplicate?).to be true
    end
  end

  context 'when user is responsible' do
    let(:user) { responsible }

    it 'permits actions based on directory access' do
      expect(subject.index?).to be true
      expect(subject.sort?).to be true
      expect(subject.add_doc_to_completeness_form?).to be true
      expect(subject.add_doc_to_completeness?).to be true
      expect(subject.add_doc_to_tagset_doc_form?).to be true
      expect(subject.add_doc_to_tagset_doc?).to be true
      expect(subject.remove?).to be true
      expect(subject.remove_doc_from_tagset_doc?).to be true
      expect(subject.bulk_delete?).to be true
      expect(subject.duplicate?).to be true
    end
  end

  context 'when user is a writer' do
    let(:user) { writer }

    it 'permits actions based on directory access' do
      expect(subject.index?).to be true
      expect(subject.sort?).to be true
      expect(subject.add_doc_to_completeness_form?).to be true
      expect(subject.add_doc_to_completeness?).to be true
      expect(subject.add_doc_to_tagset_doc_form?).to be true
      expect(subject.add_doc_to_tagset_doc?).to be true
      expect(subject.remove?).to be true
      expect(subject.remove_doc_from_tagset_doc?).to be true
      expect(subject.bulk_delete?).to be true
      expect(subject.duplicate?).to be true
    end
  end

  context 'when user is a reader' do
    let(:user) { reader }

    it 'denies actions that require write access' do
      expect(subject.index?).to be true
      expect(subject.sort?).to be true
      expect(subject.add_doc_to_completeness_form?).to be true
      expect(subject.add_doc_to_completeness?).to be true
      expect(subject.add_doc_to_tagset_doc_form?).to be true
      expect(subject.add_doc_to_tagset_doc?).to be true
      expect(subject.remove?).to be false
      expect(subject.remove_doc_from_tagset_doc?).to be false
      expect(subject.bulk_delete?).to be false
      expect(subject.duplicate?).to be false
    end
  end

  context 'when user is other' do
    let(:user) { other_user }

    it 'denies all actions' do
      expect(subject.index?).to be false
      expect(subject.sort?).to be false
      expect(subject.add_doc_to_completeness_form?).to be false
      expect(subject.add_doc_to_completeness?).to be false
      expect(subject.add_doc_to_tagset_doc_form?).to be false
      expect(subject.add_doc_to_tagset_doc?).to be false
      expect(subject.remove?).to be false
      expect(subject.remove_doc_from_tagset_doc?).to be false
      expect(subject.bulk_delete?).to be false
      expect(subject.duplicate?).to be false
    end
  end

  describe SpiritHub::DirectoriesPolicy::Scope do
    let(:scope) { Tagset.where(spirit_hubdoc: true) }
    let!(:directory1) { create(:tagset_directory, responsible: responsible.id, readers: [reader.id], writers: [writer.id], accounted: [accounted_user.id.to_s]) }
    let!(:directory2) { create(:tagset_directory) }
    let!(:directory_not_in_spirit) { create(:tagset_directory, spirit_hubdoc: false) }

    subject { described_class.new(user, scope).resolve }

    context 'when the user is an admin' do
      let(:user) { admin }

      it 'returns all spirit directories' do
        expect(subject).to match_array([directory1, directory2])
      end
    end

    context 'when the user is responsible, reader, writer or accounted' do
      it 'returns directories accessible by the user' do
        [responsible, reader, writer, accounted_user].each do |user|
          expect(described_class.new(user, scope).resolve).to match_array([directory1])
        end
      end
    end

    context 'when other_user' do
      let(:user) { other_user }

      it 'returns an empty array' do
        expect(subject).to match_array([])
      end
    end
  end
end
