# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::FavoritesPolicy do
  let(:favorite_user) { create(:user) }
  let(:other_user) { create(:user) }
  let(:admin) { create(:user, roles: ['Administrator']) }
  let(:reader) { create(:user) }
  let(:writer) { create(:user) }
  let(:responsible) { create(:user) }
  let(:tagset_doc) { create(:tagset_doc, responsible: responsible.id, readers: [reader.id.to_s], writers: [writer.id.to_s]) }
  let(:favorite) { create(:favorite, user: favorite_user, tagset: tagset_doc) }

  subject { described_class }

  permissions :index?, :sort? do
    it 'permits all users' do
      [admin, reader, writer, responsible, favorite_user].each do |user|
        expect(subject).to permit(user)
      end
    end
  end

  permissions :remove? do
    it 'permits the writers of tagset' do
      [admin, writer, responsible].each do |user|
        expect(subject).to permit(user, tagset_doc)
      end
    end

    it 'denies other users' do
      [reader, other_user, favorite_user].each do |user|
        expect(subject).not_to permit(user, tagset_doc)
      end
    end
  end

  describe SpiritHub::FavoritesPolicy::Scope do
    let(:scope) { Tagset.all }
    let(:tagset_spirit) { create(:tagset, spirit_hubdoc: true) }
    let(:tagset_not_in_spirit) { create(:tagset) }

    subject { described_class.new(user, scope) }

    describe '#resolve' do
      context 'when the user is an admin' do
        let(:user) { admin }
        it 'returns all tagsets' do
          expect(subject.resolve).to eq([tagset_doc, tagset_spirit])
        end
      end

      context 'when the user has not access to the tagset' do
        let(:user) { favorite_user }
        it 'returns empty array' do
          expect(subject.resolve).to eq([])
        end
      end

      context 'when the user has not access to the tagset' do
        let(:user) { responsible }
        it 'returns the tagset doc' do
          expect(subject.resolve).to eq([tagset_doc])
        end
      end
    end
  end
end
