# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::DirectoryCompletenessUpdater, type: :service) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  subject { described_class.call(tagset.reload) }

  let(:tagset) { create(:tagset_directory) }

  let(:create_child_directory) do
    proc do |tagset, **opts|
      child = create(:tagset_directory, **opts.except(:completed_completeness, :incomplete_completeness))

      opts[:completed_completeness]&.times do
        create(:completeness, :completed, tagset: child)
      end

      opts[:incomplete_completeness]&.times do
        create(:completeness, tagset: child)
      end

      tagset.update!(tagset_ids: tagset.tagset_ids.to_a + [child.id])

      child
    end
  end

  RSpec.shared_examples 'directory is marked as complete' do
    it 'sets the content_completeness to 100' do
      subject
      expect(tagset.reload.content_completeness).to eq 100
    end

    it 'sets the current_state to complete' do
      subject
      expect(tagset.reload.current_state).to eq described_class::STATE_COMPLETE
    end
  end

  RSpec.shared_examples 'directory is marked as incomplete' do |content_completeness:|
    it 'sets the content_completeness to the correct percentage' do
      subject
      expect(tagset.reload.content_completeness).to eq content_completeness
    end

    it 'sets the current_state to incomplete' do
      subject
      expect(tagset.reload.current_state).to eq described_class::STATE_INCOMPLETE
    end
  end

  RSpec.shared_examples 'directory is unmarked' do
    it 'sets the content_completeness to nil' do
      subject
      expect(tagset.reload.content_completeness).to be_nil
    end

    it 'sets the current_state to nil' do
      subject
      expect(tagset.reload.current_state).to be_nil
    end
  end

  context 'when passing a non directory tagset' do
    let(:tagset) { create(:tagset_doc) }

    it 'raises an error' do
      expect { subject }.to raise_error ArgumentError
    end
  end

  context 'with no child directories' do
    before do
      tagset.children_tagsets.each(&:destroy!)
      tagset.reload
    end

    include_examples 'directory is unmarked'
  end

  context 'with child directories without completeness' do
    let!(:child1) { create_child_directory.call(tagset) }
    let!(:child2) { create_child_directory.call(tagset) }

    include_examples 'directory is unmarked'
  end

  context 'with child directories and some incomplete' do
    let!(:child1) { create_child_directory.call(tagset) }
    let!(:child2) { create_child_directory.call(tagset, completed_completeness: 2, incomplete_completeness: 1) }
    let!(:child3) { create_child_directory.call(tagset, completed_completeness: 1, incomplete_completeness: 8) }

    include_examples 'directory is marked as incomplete', content_completeness: 25
  end

  context 'with child directories all complete' do
    let!(:child1) { create_child_directory.call(tagset, completed_completeness: 1) }
    let!(:child2) { create_child_directory.call(tagset, completed_completeness: 1) }

    include_examples 'directory is marked as complete'

    context 'when updating child completeness' do
      before do
        SpiritHub::Completeness.where(tagset_id: child1.id).first.update(object_ids: [])
      end

      include_examples 'directory is marked as incomplete', content_completeness: 50
    end
  end

  context 'with completeness on the directory' do
    let!(:completeness) { create(:completeness, tagset:) }

    include_examples 'directory is marked as incomplete', content_completeness: 0

    context 'when completeness is completed' do
      before do
        completeness.update(object_ids: [create(:doc).id])
      end

      include_examples 'directory is marked as complete'
    end
  end
end
