# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::OperationTreeModel, type: :service) do
  let!(:admin) { create(:user, roles: ['Administrator']) }

  describe 'create_batch' do
    subject { described_class.create_batch }

    it 'should create all record' do
      subject
      batch = Batch.find_by(name: 'Classification plans models')
      expect(batch.tagset_id).to be_present
      expect(batch.tagset).to be_present
      expect(Category.where(name: 'Dépôt', cycle_id: batch.tagset.blocset_id)).to be_present
      expect(Category.where(name: 'Fichiers valides', cycle_id: batch.tagset.blocset_id)).to be_present
      expect(Category.where(name: 'Fichiers non valides', cycle_id: batch.tagset.blocset_id)).to be_present
      expect(batch.inline_script).to eq("SpiritHub::OperationTreeModel.validate_models\n")
    end
  end

  describe 'validate_models' do
    let!(:doc_csv) { create(:doc, title: 'doc.csv', file_path: 'packages/spirit_hub/spec/fixtures/operation_trees/sample1.csv') }

    subject { described_class.validate_models }

    before do
      SpiritHub::HubdocCycle.import_cycle_from_json
      described_class.create_batch
      batch = Batch.find_by(name: 'Classification plans models')
      deposit_category = Category.find_by(name: 'Dépôt')
      doc_csv.set_category(batch.tagset_id, deposit_category.id)
    end

    it 'should create all record' do
      subject
      expect(Group.all.pluck(:name)).to match_array(%w[
                                                      spirit-immo_alsace_responsable-maitrise
                                                      spirit-immo_alsace_responsable-conformité
                                                      spirit-immo_alsace_responsable-terrain
                                                      spirit-immo_alsace_service_juridique
                                                      spirit-immo_alsace_responsable-juridique
                                                    ])
      expect(Tagset.all.pluck(:title)).to match_array(['Classification plans models',
                                                       'Descriptif du foncier',
                                                       'Existant',
                                                       'Etudes réalisées',
                                                       'Acquisition Propriétaire XX',
                                                       'Terrain',
                                                       'Projet',
                                                       'NF Logement',
                                                       'Financier',
                                                       "Maîtrise d'ouvrage",
                                                       "Lancement de l'exécution",
                                                       'Concessionnaires',
                                                       'Prestataires techniques',
                                                       "Appel d'offre - DCE",
                                                       "Maîtrise d'œuvre d'exécution",
                                                       'Contentieux Entreprises',
                                                       'Contentieux Clients',
                                                       'Contentieux Tiers',
                                                       'Déclarations de sinistres',
                                                       'Juridique et gestion des risques',
                                                       'Projet doc'])
      expect(Doc.all.pluck(:title)).to match_array([
                                                     'doc.csv',
                                                     'Conformité',
                                                     'Plan de situation',
                                                     'Etat des lieux initial (photos + plans + rapport)',
                                                     "Extrait cadastral - documents d'arpentage (plan géomètre)",
                                                     'Règlement applicable (PLU, POS, ZAC, Servitude, PPRI, MH, etc…)',
                                                     'Etude des sols préliminaire',
                                                     'Etude des sols complémentaire',
                                                     'Diagnostic pollution',
                                                     'Attestation(s) de dépollution',
                                                     'Rapport(s) amiante',
                                                     'Rapport(s) plomb',
                                                     'Etat(s) parasitaire(s)',
                                                     'Etat locatif (bail, loyer, bilans,…)',
                                                     'Offre acceptée par le propriétaire',
                                                     'Promesse, avenant et annexes',
                                                     'Convention(s) propriétaire et-ou locataire (servitude, éviction,…)',
                                                     "Attestation notarié d'aquisition",
                                                     'Acte authentique',
                                                     'Convention(s) collectivités',
                                                     'CCTP',
                                                     'CCTP',
                                                     'Assurance des entreprises',
                                                     'Opérations préalables à la réception (OPR)',
                                                     "Tableau récapitulatif des contentieux de l'opération"
                                                   ])
    end

    it 'should have a tree with the right values' do
      subject

      terrain = Tagset.find_by(title: 'Terrain')
      expect(Tagset.where(:id.in => terrain.tagset_ids).pluck(:title)).to match_array(['Descriptif du foncier', 'Acquisition Propriétaire XX', 'Etudes réalisées'])
      expect(Doc.find(terrain.doc_ids.first).title).to eq('Convention(s) collectivités')
      reader_group_ids = terrain.readers.select { |r| r.include?('g_') }.map { |r| r.gsub('g_', '') }
      expect(Group.where(:id.in => reader_group_ids).pluck(:name)).to match_array(%w[spirit-immo_alsace_responsable-terrain])
      writer_group_ids = terrain.writers.select { |r| r.include?('g_') }.map { |r| r.gsub('g_', '') }
      expect(Group.where(:id.in => writer_group_ids).pluck(:name)).to match_array(%w[spirit-immo_alsace_responsable-maitrise spirit-immo_alsace_responsable-terrain])
      expect(terrain.superfields[:number_of_months]).to be_nil
      expect(terrain.superfields[:chapter]).to eq('Opération de promotion')
      expect(terrain.superfields[:sub_chapter]).to eq('3 - MONTAGE')
      expect(terrain.superfields[:document_type]).to be_nil
    end
  end
end
