# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::TagsetDuplicater, type: :service) do
  describe 'can_duplicate?' do
    subject { described_class.can_duplicate?(tagset) }
    context 'when max_duplicates is blank' do
      let!(:tagset) { create(:tagset) }
      it 'returns false' do
        expect(subject).to(be_falsey)
      end
    end

    context 'when max_duplicates is *' do
      let!(:tagset) { create(:tagset, max_duplicates: '*', existing_duplicates: [1, 2, 3, 4]) }

      it 'returns true' do
        expect(subject).to(be_truthy)
      end
    end

    context 'when max_duplicates is a number' do
      let!(:tagset) { create(:tagset, max_duplicates: '2', existing_duplicates: [1]) }

      it 'returns true' do
        expect(subject).to(be_truthy)
      end
    end

    context 'when max_duplicates is a number and max_duplicates >= existing_duplicates' do
      let!(:tagset) { create(:tagset, max_duplicates: '2', existing_duplicates: [1, 2]) }

      it 'returns false' do
        expect(subject).to(be_falsey)
      end
    end
  end

  describe 'call' do
    let!(:operation) { create(:operation, title: 'Operation 1') }
    let!(:phase) { create(:phase, title: 'Phase 1', operation:) }
    let!(:tagset_default_attributes) { { operation_id: operation.id, readers: ['readers'], writers: ['writers'], phase_id: phase.id } }
    let!(:sub_sub_tagset1) { create(:tagset_directory, tagset_default_attributes.merge(title: 'SubSub Tagset 1')) }
    let!(:sub_sub_completeness1) { create(:completeness, title: 'SubSub Completeness 1', tagset_id: sub_sub_tagset1.id) }
    let!(:sub_tagset1) { create(:tagset_directory, tagset_default_attributes.merge(title: 'Sub Tagset 1', tagset_ids: [sub_sub_tagset1.id])) }
    let!(:sub_completeness1) { create(:completeness, title: 'Sub Completeness 1', tagset_id: sub_tagset1.id) }
    let!(:sub_completeness2) { create(:completeness, title: 'Sub Completeness 2', tagset_id: sub_tagset1.id) }
    let!(:sub_tagset2) { create(:tagset_directory, tagset_default_attributes.merge(title: 'Sub Tagset 2', tagset_ids: [])) }
    let!(:tagset) { create(:tagset_directory, tagset_default_attributes.merge(title: 'Tagset', tagset_ids: [sub_tagset1.id, sub_tagset2.id], max_duplicates: '*')) }
    let!(:parent_tagset) { create(:tagset_directory, tagset_default_attributes.merge(title: 'Parent Tagset', tagset_ids: [tagset.id])) }

    subject { described_class.call(tagset.reload) }

    it 'create tagsets' do
      expect { subject }.to(change { Tagset.count }.by(4))
    end

    it 'create completeness' do
      expect { subject }.to(change { SpiritHub::Completeness.count }.by(3))
    end

    it 'create the right tagset tree' do
      subject

      new_tagset = Tagset.find_by(:id.ne => tagset.id, title: 'Tagset')
      expect(new_tagset.tagset_ids.length).to eq(2)

      described_class::TAGSET_DUPLICATED_ATTRIBUTES.each do |attribute|
        expect(new_tagset.send(attribute)).to eq(tagset.send(attribute))
      end

      new_sub_tagset1 = Tagset.find_by(:id.ne => sub_tagset1.id, title: 'Sub Tagset 1')
      expect(new_sub_tagset1.completeness.length).to eq(2)

      new_completeness = new_sub_tagset1.completeness.first
      described_class::COMPLETENESS_DUPLICATED_ATTRIBUTES.each do |attribute|
        expect(new_completeness.send(attribute)).to eq(sub_completeness1.send(attribute))
      end

      expect(parent_tagset.reload.tagset_ids.length).to eq(2)
      expect(tagset.reload.existing_duplicates).to include(new_tagset.id.to_s)
    end
  end
end
