# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::TagsetNotifier, type: :service) do
  subject { described_class.call(tagset, current_user) }

  describe 'call' do
    let(:current_user) { create(:user) }
    let(:user) { create(:user) }

    context 'when the tagset is a tagset_doc' do
      context 'when tagset does not have notifiable_ids' do
        let(:tagset) { create(:tagset_doc, notifiable_ids: []) }

        it { expect { subject }.not_to have_enqueued_job(ActionMailer::MailDeliveryJob) }
      end

      context 'when tagset has notifiable_ids with user related' do
        let(:tagset) { create(:tagset_doc, notifiable_ids: ["u_#{user.id}"]) }

        it {
          expect do
            subject
          end.to have_enqueued_job(ActionMailer::MailDeliveryJob).with('SpiritHub::TagsetsMailer', 'document_uploaded', 'deliver_now',
                                                                       args: [{ tagset_id: tagset.id.to_s, emails: [user.email] }]).once
        }
      end

      context 'when tagset has notifiable_ids with contact related' do
        let(:tagset) { create(:tagset_doc, notifiable_ids: ["c_#{contact.id}"]) }

        let(:contact) { create(:contact, current_user_id: user.id, title: 'my-contact@example.com') }

        it {
          expect do
            subject
          end.to have_enqueued_job(ActionMailer::MailDeliveryJob).with('SpiritHub::TagsetsMailer', 'document_uploaded', 'deliver_now',
                                                                       args: [{ tagset_id: tagset.id.to_s, emails: [contact.title] }]).once
        }
      end

      context 'when tagset has notifiable_ids with user and contact related' do
        let(:tagset) { create(:tagset_doc, notifiable_ids: ["c_#{contact.id}", "u_#{user.id}"]) }

        let(:contact) { create(:contact, current_user_id: user.id, title: 'my-contact@example.com') }

        it {
          expect do
            subject
          end.to have_enqueued_job(ActionMailer::MailDeliveryJob).with('SpiritHub::TagsetsMailer', 'document_uploaded', 'deliver_now',
                                                                       args: [{ tagset_id: tagset.id.to_s, emails: [contact.title, user.email] }]).once
        }
      end

      context 'when tagset has notifiable_ids with user identical to current user' do
        let(:tagset) { create(:tagset_doc, notifiable_ids: ["u_#{user.id}"]) }
        let(:current_user) { user }

        it { expect { subject }.not_to have_enqueued_job(ActionMailer::MailDeliveryJob) }
      end

      context 'when parents tagset have notifiable_ids' do
        let(:root_contact) { create(:contact, current_user_id: user.id, title: 'root-contact@example.com') }
        let(:root) { create(:tagset_directory, notifiable_ids: ["c_#{root_contact.id}"]) }
        let(:child_contact) { create(:contact, current_user_id: user.id, title: 'child-contact@example.com') }
        let(:child) { create(:tagset_directory, parent_tagset_ids: [root.id], notifiable_ids: ["c_#{child_contact.id}"]) }
        let(:tagset) { create(:tagset_doc, parent_tagset_ids: [child.id]) }

        it {
          expect do
            subject
          end.to have_enqueued_job(ActionMailer::MailDeliveryJob).with('SpiritHub::TagsetsMailer', 'document_uploaded', 'deliver_now',
                                                                       args: [{ tagset_id: tagset.id.to_s, emails: [root_contact.title, child_contact.title] }]).once
        }
      end

      context 'when tagset has notifiable_ids with no prefix' do
        let(:tagset) { create(:tagset_doc, notifiable_ids: [user.id.to_s]) }

        it { expect { subject }.to raise_error(ArgumentError, "Invalid id: #{user.id}") }
      end
    end

    context 'when tagset is a directory' do
      let(:tagset) { create(:tagset_directory, notifiable_ids: ["u_#{user.id}"]) }

      it { expect { subject }.not_to have_enqueued_job(ActionMailer::MailDeliveryJob) }
    end
  end
end
