# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::TagsetPurger, type: :service) do
  describe 'call' do
    let(:tagset) { create(:tagset_doc) }
    let(:user) { create(:user) }
    subject { described_class.call(tagset, user) }

    it 'destroys the record' do
      expect(tagset).to be_persisted
      subject
      expect(tagset).not_to be_persisted
      expect(Tagset.find(tagset.id)).to be_nil
    end

    it 'add the action to the journal' do
      allow(SpiritHub::JournalEntry).to receive(:create)
      subject

      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_DELETED',
        tagset_id: tagset.id,
        user_id: user.id,
        action: "Document #{tagset.title} supprimé par #{user.full_name}",
        action_msg: nil,
        opts: instance_of(Hash)
      ).once
    end

    context 'when the tagset is a child of another tagset' do
      let!(:tagset_parent1) { create(:tagset_directory, tagset_ids: ['other_tagset_id', tagset.id.to_s]) }
      let!(:tagset_parent2) { create(:tagset_directory, tagset_ids: [tagset.id.to_s]) }

      it 'clears the references from the parents' do
        expect(tagset_parent1.tagset_ids).to include(tagset.id.to_s)
        expect(tagset_parent2.tagset_ids).to include(tagset.id.to_s)

        subject

        expect(tagset_parent1.reload.tagset_ids).to eq ['other_tagset_id']
        expect(tagset_parent2.reload.tagset_ids).to eq []
      end
    end

    context 'when the tagset is a associated to a completeness rule' do
      let!(:completeness1) { create(:completeness, tagset: tagset_owner, object_ids: ['other_tagset_id', tagset.id]) }
      let!(:completeness2) { create(:completeness, tagset: tagset_owner, object_ids: [tagset.id]) }
      let(:tagset_owner) { create(:tagset_directory) }

      it 'clears the references from the parents' do
        expect(completeness1.object_ids).to include(tagset.id)
        expect(completeness2.object_ids).to include(tagset.id)

        subject

        expect(completeness1.reload.object_ids).to eq ['other_tagset_id']
        expect(completeness2.reload.object_ids).to eq []
      end
    end
  end
end
