# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetRequestSignature, type: :service do
  subject { described_class.call(**params) }

  let(:params) do
    {
      tagset:,
      doc_ids:,
      recipients:,
      current_user: user3
    }
  end

  let(:tagset) { create(:tagset_doc, :with_file) }
  let(:doc_ids) { tagset.docs.pluck(:id).map(&:to_s) }
  let(:recipients) { [user1.id.to_s, user2.id.to_s] }

  let(:user1) { create(:user) }
  let(:user2) { create(:user) }
  let(:user3) { create(:user) }

  let!(:to_sign_state) { create(:state, :default_cycle, key: 'to_sign') }

  it 'changes the state of the Tagset accordingly' do
    expect(tagset.state).to_not eq to_sign_state
    subject
    expect(tagset.reload.state).to eq to_sign_state
  end

  it 'overwrites the tagset accounted users accordingly' do
    tagset.update!(accounted: [user3.id.to_s])
    tagset.reload

    subject

    expect(tagset.accounted.map(&:to_s)).to match_array(recipients + [user3.id.to_s])
  end

  it 'sends the signature request async to the recipients' do
    expect(SpiritHub::Docs::RequestSignatureJob).to receive(:perform_later).with(
      doc_ids,
      [user1.id.to_s, user2.id.to_s],
      user3.id.to_s
    )

    subject
  end

  it 'add the action to the journal' do
    allow(SpiritHub::JournalEntry).to receive(:create)

    subject

    expect(SpiritHub::JournalEntry).to have_received(:create).with(
      code: 'DOC_SENT_TO_SIGN',
      tagset_id: tagset.id,
      user_id: user3.id,
      action: "Document #{tagset.title} envoyé à #{user1.full_name} et #{user2.full_name} par #{user3.full_name} pour signature.",
      action_msg: nil,
      opts: instance_of(Hash)
    ).once
  end

  it 'returns true' do
    expect(subject).to eq true
  end

  context 'when passing comments' do
    let(:comments) { 'Please sign asap!' }

    before { params[:comments] = comments }

    it 'add the action with the correct msg' do
      allow(SpiritHub::JournalEntry).to receive(:create)

      subject

      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        hash_including(
          action: "Document #{tagset.title} envoyé à #{user1.full_name} et #{user2.full_name} par #{user3.full_name} pour signature.",
          action_msg: 'Commentaire: Please sign asap!'
        )
      ).once
    end
  end

  context 'when not passing the target doc id' do
    before { params[:doc_ids] = nil }

    it 'does nothing' do
      allow(SpiritHub::JournalEntry).to receive(:create)
      allow(SpiritHub::Docs::RequestSignatureJob).to receive(:perform_later)

      expect(subject).to eq false
      expect(tagset.reload.state).to_not eq to_sign_state
      expect(tagset.accounted.map(&:to_s)).to_not match_array recipients
      expect(SpiritHub::JournalEntry).to_not have_received(:create).with(hash_including(code: 'DOC_SENT_TO_SIGN'))
      expect(SpiritHub::Docs::RequestSignatureJob).to_not have_received(:perform_later)
    end
  end

  context 'when passing a target doc id not belonging to the tagset' do
    before { params[:doc_ids] = [create(:doc).id] }

    it 'does nothing' do
      allow(SpiritHub::JournalEntry).to receive(:create)
      allow(SpiritHub::Docs::RequestSignatureJob).to receive(:perform_later)

      expect(subject).to eq false
      expect(tagset.reload.state).to_not eq to_sign_state
      expect(tagset.accounted.map(&:to_s)).to_not match_array recipients
      expect(SpiritHub::JournalEntry).to_not have_received(:create).with(hash_including(code: 'DOC_SENT_TO_SIGN'))
      expect(SpiritHub::Docs::RequestSignatureJob).to_not have_received(:perform_later)
    end
  end
end
