# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetShare, type: :service do
  subject { described_class.call(**params) }

  let(:tagset) { create(:tagset_doc, :with_file) }

  let(:user1) { create(:user) }
  let(:user2) { create(:user) }
  let(:user3) { create(:user) }
  let(:user4) { create(:user) }

  let(:message_subject) { Faker::Lorem.sentence }
  let(:message) { Faker::Lorem.sentence }

  before { SpiritHub::HubdocCycle.import_cycle_from_json }
  before { allow(SpiritHub::TagsetsMailer).to receive(:share).and_call_original }
  before { allow(SpiritHub::JournalEntry).to receive(:create) }
  before { allow(LogJob).to receive(:perform_async) }

  context 'when share action' do
    let(:params) do
      {
        tagset:,
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4
      }
    end

    it 'shares the doc to the user' do
      Upload.destroy_all
      Doc.where(type: 'email').destroy_all

      subject

      # creates an Upload record
      expect(upload_created = Upload.last).to be_present
      expect(upload_created.user_id).to eq user4.id
      expect(upload_created.doc_ids).to be_present
      expect(upload_created.doc_ids).to match_array tagset.doc_ids
      expect(upload_created.doc_id).to be_nil

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id])
      expect(tagset.informed_main).to match_array([user1.id])
      expect(tagset.informed_cc).to match_array([user2.id])
      expect(tagset.informed_bcc).to match_array([user3.id])
      expect(tagset.current_state).to eq('Partagé')
      expect(tagset.state.key).to eq('share')

      # creates the Doc(type=email) record
      expect(doc_email = Doc.where(type: 'email').last).to be_present
      expect(doc_email.user).to eq user4
      expect(doc_email.title).to eq message_subject
      expect(doc_email.body).to include message
      expect(doc_email.superfields['mail_from']).to eq user4.email
      expect(doc_email.superfields['mail_to']).to eq user1.email
      expect(doc_email.superfields['mail_cc']).to eq user2.email
      expect(doc_email.superfields['mail_bcc']).to eq user3.email

      # creates a comment record
      comment = InstantMessage.last
      expect(comment).to be_present
      expect(comment.channel.key).to eq "upload_comments_#{upload_created.id}"
      expect(comment.text).to eq message
      expect(comment.user).to eq user4
      expect(comment.channel.upload_id).to eq upload_created.id

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        upload: upload_created.id.to_s,
        subject: message_subject,
        message:,
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )

      # journal_entries
      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_SHARED',
        tagset_id: tagset.id,
        user_id: user4.id,
        action: "Document #{tagset.title} partagé par #{user4.full_name} à #{user1.full_name}, #{user2.full_name} et #{user3.full_name}",
        action_msg: nil,
        opts: hash_including(doc_ids: upload_created.doc_ids)
      ).once

      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'SHARE_LINK_CREATED',
        tagset_id: tagset.id,
        user_id: user4.id,
        action: "Lien de partage: http://#{APP_CONFIG['app_host']}/spirit_hub/uploads/#{upload_created.id}",
        action_msg: nil,
        opts: hash_including(
          upload_id: upload_created.id,
          doc_ids: upload_created.doc_ids
        )
      ).once
    end

    it 'delivers the share email' do
      share_email = double(
        deliver_later: nil,
        subject: message_subject,
        body: message,
        from: [user4.email],
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )
      allow(SpiritHub::TagsetsMailer).to receive(:share).and_return(share_email)

      subject

      expect(share_email).to have_received(:deliver_later)
    end

    context 'when passing the specific doc ids to share' do
      let(:tagset) { create(:tagset_doc, doc_ids: docs.map(&:id)) }
      let(:docs) { create_list(:doc, 3) }
      let(:external_doc) { create(:doc) }

      before { params[:doc_ids] = [docs[0].id, docs[2].id, external_doc.id] }

      it 'shares only these docs and only the ones that belong to the tagset' do
        Upload.destroy_all
        Doc.where(type: 'email').destroy_all

        subject

        expected_shared_doc_ids = [docs[0].id.to_s, docs[2].id.to_s] # docs[1] and external_doc are left out

        expect(upload_created = Upload.last).to be_present
        expect(upload_created.doc_ids).to be_present
        expect(upload_created.doc_ids).to match_array expected_shared_doc_ids

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          hash_including(
            code: 'MULTIPLE_DOC_SHARED',
            tagset_id: tagset.id,
            opts: hash_including(
              doc_ids: expected_shared_doc_ids
            )
          )
        ).once

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          hash_including(
            code: 'SHARE_LINK_CREATED',
            tagset_id: tagset.id,
            opts: hash_including(
              doc_ids: expected_shared_doc_ids
            )
          )
        ).once
      end
    end

    context 'when using a tagset_directory' do
      let(:tagset_doc) { create(:tagset_doc, :with_file) }
      let(:tagset) { create(:tagset_directory, tagset_ids: [tagset_doc.id]) }

      it 'udpate the informed attribute of the tagset_doc' do
        subject

        expect(tagset_doc.reload.informed).to match_array([user1.id, user2.id, user3.id])
        expect(tagset_doc.informed_main).to match_array([user1.id])
        expect(tagset_doc.informed_cc).to match_array([user2.id])
        expect(tagset_doc.informed_bcc).to match_array([user3.id])
      end
    end

    context 'when passing a group as one of the main recipients' do
      let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
      let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

      let(:user5) { create(:user) }
      let(:user6) { create(:user) }

      before { params[:to] += [group_id_prefixed] }

      it 'shares the doc with the users within that group' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
        expect(tagset.informed_main).to match_array([user1.id, user5.id, user6.id])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            to: [user1.email, user5.email, user6.email]
          )
        )
      end

      context "when group is empty and it's the sole recipient" do
        let(:group) { create(:group, users_ids: []) }

        before { params[:to] = [group_id_prefixed] }

        it 'does not send any email' do
          subject
          expect(SpiritHub::TagsetsMailer).to_not have_received(:share)
        end
      end
    end

    context 'when passing a group as one of the CC recipients' do
      before { params[:cc] += [group_id_prefixed] }

      let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
      let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

      let(:user5) { create(:user) }
      let(:user6) { create(:user) }

      it 'shares the doc with the users within that group' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
        expect(tagset.informed_cc).to match_array([user2.id, user5.id, user6.id])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            cc: [user2.email, user5.email, user6.email]
          )
        )
      end
    end

    context 'when passing a group as one of the BCC recipients' do
      before { params[:bcc] += [group_id_prefixed] }

      let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
      let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

      let(:user5) { create(:user) }
      let(:user6) { create(:user) }

      it 'shares the doc with the users within that group' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
        expect(tagset.informed_bcc).to match_array([user3.id, user5.id, user6.id])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            bcc: [user3.email, user5.email, user6.email]
          )
        )
      end
    end

    context 'when passing a contact as one of the main recipients' do
      before { params[:to] += [contact_id_prefixed] }

      let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
      let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
      let(:contact) { create(:contact) }

      it 'shares the doc with that contact' do
        expect(contact.reload.email).to be_present

        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            to: [user1.email, contact.email]
          )
        )
      end
    end

    context 'when passing a contact as one of the CC recipients' do
      before { params[:cc] += [contact_id_prefixed] }

      let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
      let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
      let(:contact) { create(:contact) }

      it 'shares the doc with that contact' do
        expect(contact.reload.email).to be_present

        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_cc).to match_array([user2.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            cc: [user2.email, contact.email]
          )
        )
      end
    end

    context 'when passing a contact as one of the BCC recipients' do
      before { params[:bcc] += [contact_id_prefixed] }

      let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
      let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
      let(:contact) { create(:contact) }

      it 'shares the doc with that contact' do
        expect(contact.reload.email).to be_present

        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_bcc).to match_array([user3.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            bcc: [user3.email, contact.email]
          )
        )
      end
    end

    context 'when passing a NEW contact as one of the main recipients' do
      before { params[:to] += ['jack.sparrow@gmail.com'] }

      let(:contact_created) { Doc.where(type: 'contact').last }
      let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

      it 'creates a new Doc(contact) record' do
        expect { subject }.to change { Doc.where(type: 'contact').count }.by(1)

        expect(contact_created).to be_present
        expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.current_user_id).to eq user4.id
      end

      it 'shares the doc with that contact' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            to: [user1.email, contact_created.email]
          )
        )
      end

      it 'creates the journal entry log with the correct opts' do
        subject

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          code: 'DOC_SHARED',
          tagset_id: tagset.id,
          user_id: user4.id,
          action: instance_of(String),
          action_msg: nil,
          opts: hash_including(
            recipients: [user1.id.to_s, "c_#{contact_created.id}", user2.id.to_s, user3.id.to_s],
            to: [user1.id.to_s, "c_#{contact_created.id}"],
            cc: [user2.id.to_s],
            bcc: [user3.id.to_s]
          )
        ).once
      end

      context 'but in the meantime that new contact got persisted in the DB somewhere else' do
        let(:contact_created) { create(:contact, email: 'jack.sparrow@gmail.com') }
        let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

        before { contact_created }

        it 'reuses that Doc(contact) record and does not create a new/duplicate one' do
          expect { subject }.to_not(change { Doc.where(type: 'contact').count })
        end

        it 'shares the doc with that contact' do
          subject

          # updates the informed fields accordingly
          expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
          expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

          # sends the share email
          expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
            hash_including(
              to: [user1.email, contact_created.email]
            )
          )
        end
      end
    end

    context 'when passing a NEW contact as one of the CC recipients' do
      before { params[:cc] += ['jack.sparrow@gmail.com'] }

      let(:contact_created) { Doc.where(type: 'contact').last }
      let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

      it 'creates a new Doc(contact) record' do
        expect { subject }.to change { Doc.where(type: 'contact').count }.by(1)

        expect(contact_created).to be_present
        expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.current_user_id).to eq user4.id
      end

      it 'shares the doc with that contact' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_cc).to match_array([user2.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            cc: [user2.email, contact_created.email]
          )
        )
      end
    end

    context 'when passing a NEW contact as one of the BCC recipients' do
      before { params[:bcc] += ['jack.sparrow@gmail.com'] }

      let(:contact_created) { Doc.where(type: 'contact').last }
      let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

      it 'creates a new Doc(contact) record' do
        expect { subject }.to change { Doc.where(type: 'contact').count }.by(1)

        expect(contact_created).to be_present
        expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
        expect(contact_created.current_user_id).to eq user4.id
      end

      it 'shares the doc with that contact' do
        subject

        # updates the informed fields accordingly
        expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
        expect(tagset.informed_bcc).to match_array([user3.id, contact_id_tagged])

        # sends the share email
        expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
          hash_including(
            bcc: [user3.email, contact_created.email]
          )
        )
      end
    end

    context 'when passing attached files' do
      let(:attached_files) do
        [
          ActionDispatch::Http::UploadedFile.new(
            tempfile: StringIO.new('File content 1'),
            filename: 'document1.pdf',
            type: 'application/pdf'
          ),
          ActionDispatch::Http::UploadedFile.new(
            tempfile: StringIO.new('File content 2'),
            filename: 'document2.pdf',
            type: 'application/pdf'
          )
        ]
      end

      let(:tagset) { create(:tagset_doc, :with_parents) }
      let(:parents_from_original_shared_tagset_doc) { tagset.reload.parents }
      let(:attached_tagset_doc1) { create(:tagset_doc, :with_parents) }
      let(:attached_tagset_doc2) { create(:tagset_doc, :with_parents) }

      before do
        params[:attachments] = attached_files
        params[:attached_tagset_ids] = [attached_tagset_doc1.id, attached_tagset_doc2.id]
      end

      it 'captures each of the attached files as a tagset doc with the correct state' do
        allow(Doc).to receive(:create_from_file!).and_call_original

        expect(parents_from_original_shared_tagset_doc).to be_present

        subject

        expect(Doc).to have_received(:create_from_file!).with(attached_files[0], user4)
        expect(Doc).to have_received(:create_from_file!).with(attached_files[1], user4)

        attached_files.each do |file|
          tagset_doc = Tagset.find_by(title: file.original_filename)
          expect(tagset_doc).to be_present
          expect(tagset_doc.type).to eq 'document'
          expect(tagset_doc.docs.length).to eq 1
          expect(tagset_doc.docs.first.title).to eq file.original_filename
          expect(tagset_doc.parents).to be_present
          expect(tagset_doc.parents).to match_array parents_from_original_shared_tagset_doc
          expect(tagset_doc.cycle_id).to eq SpiritHub::HubdocCycle.default_cycle.id
          expect(tagset_doc.state.key).to eq('done')
        end
      end

      it 'does not update any of the existing attached tagset docs attributes' do
        expect(parents_from_original_shared_tagset_doc).to be_present

        subject

        expect(attached_tagset_doc1.reload.parents).to_not match_array(parents_from_original_shared_tagset_doc)
        expect(attached_tagset_doc2.reload.parents).to_not match_array(parents_from_original_shared_tagset_doc)
        expect(attached_tagset_doc1.state&.key).to_not eq('done')
        expect(attached_tagset_doc2.state&.key).to_not eq('done')
      end

      it 'shares the doc including the attachments to the user' do
        Upload.destroy_all

        subject

        attached_doc1 = attached_tagset_doc1.doc_ids
        attached_doc2 = attached_tagset_doc2.doc_ids
        attached_doc3 = Doc.find_by!(title: 'document1.pdf').id
        attached_doc4 = Doc.find_by(title: 'document2.pdf').id

        # creates an Upload record
        expect(upload_created = Upload.last).to be_present
        expect(upload_created.user_id).to eq user4.id
        expect(upload_created.doc_ids).to be_present
        expect(upload_created.doc_ids).to match_array [*tagset.doc_ids, *attached_doc1, *attached_doc2, attached_doc3, attached_doc4]
      end
    end

    context 'when passing a blank message' do
      let(:message) { '' }

      it 'does not create a comment record' do
        expect { subject }.not_to(change { InstantMessage.count })
      end
    end
  end

  context 'when respond action' do
    let(:params) do
      {
        tagset:,
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4,
        action: :respond
      }
    end

    it 'shares the doc to the user' do
      Upload.destroy_all
      Doc.where(type: 'email').destroy_all

      subject

      # creates an Upload record
      expect(upload_created = Upload.last).to be_present
      expect(upload_created.user_id).to eq user4.id
      expect(upload_created.doc_ids).to be_present
      expect(upload_created.doc_ids).to match_array tagset.doc_ids
      expect(upload_created.doc_id).to be_nil

      # creates the Doc(type=email) record
      expect(doc_email = Doc.where(type: 'email').last).to be_present
      expect(doc_email.user).to eq user4
      expect(doc_email.title).to eq message_subject
      expect(doc_email.body).to include message
      expect(doc_email.superfields['mail_from']).to eq user4.email
      expect(doc_email.superfields['mail_to']).to eq user1.email
      expect(doc_email.superfields['mail_cc']).to eq user2.email
      expect(doc_email.superfields['mail_bcc']).to eq user3.email

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        upload: upload_created.id.to_s,
        subject: message_subject,
        message:,
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )

      # journal_entries
      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_RESPONDED',
        tagset_id: tagset.id,
        user_id: user4.id,
        action: "Réponse effectuée pour #{tagset.title} par #{user4.full_name}",
        action_msg: nil,
        opts: instance_of(Hash)
      ).once
    end

    it 'delivers the share email' do
      share_email = double(
        deliver_later: nil,
        subject: message_subject,
        body: message,
        from: [user4.email],
        to: [user1.email],
        cc: [user2.email],
        bcc: [user3.email]
      )
      allow(SpiritHub::TagsetsMailer).to receive(:share).and_return(share_email)

      subject

      expect(share_email).to have_received(:deliver_later)
    end
  end
end
