# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetSplitter, type: :service do
  before { SpiritHub::HubdocCycle.import_cycle_from_json }

  subject { described_class.call(**params) }

  let(:user) { create(:user) }
  let(:tagset) { create(:tagset_doc, :with_file, filename: 'pdf_sample.pdf', responsible: user.id.to_s) }
  let(:params) { { tagset:, user:, per_page: 2 } }

  it 'raises an error if tagset is not provided' do
    expect { described_class.call(user:) }.to raise_error(ArgumentError, 'tagset is required')
  end

  it 'raises an error if user is not provided' do
    expect { described_class.call(tagset:) }.to raise_error(ArgumentError, 'user is required')
  end

  it 'raises an error if tagset has more than one doc' do
    tagset.set(doc_ids: [1, 2])
    expect { subject }.to raise_error(StandardError, 'tagset must have only one doc')
  end

  it 'raises an error if tagset doc is not a PDF' do
    tagset.docs.first.set(file_ext: 'jpg')
    expect { subject }.to raise_error(StandardError, 'tagset doc must be a PDF')
  end

  it 'for unclassified tagset' do
    new_tagset_doc = subject

    expect(new_tagset_doc.doc_ids.length).to eq(3)
    expect(new_tagset_doc.title).to eq("#{tagset.title} (découpé)")
    expect(new_tagset_doc.parents).to eq([])
    expect(new_tagset_doc.current_state).to eq('À classer')
    expect(Dir.glob(Rails.root.join('tmp', "*#{tagset.title}*")).size).to eq(0)
    expect(SpiritHub::JournalEntry.find_by(action: /Document #{tagset.title} découpé par/)).to be_truthy
  end

  it 'for classified tagset' do
    directory = create(:tagset_directory)
    tagset.add_tagset_doc_to_tagset(directory, user)
    tagset.reload
    new_tagset_doc = subject

    expect(new_tagset_doc.reload.doc_ids.length).to eq(3)
    expect(new_tagset_doc.parents.length).to eq(1)
    expect(new_tagset_doc.current_state).to eq('Classé')
  end

  it 'when is the favorite tagset' do
    Favorite.create(user:, tagset:)
    new_tagset_doc = subject

    expect(Favorite.where(user:, tagset: new_tagset_doc).exists?).to be_truthy
  end
end
